function [IApp,CtrlFF,StimFF] = Func_FiringFrequencyCurve(Func,EXP,TH,tTransient,IAppBoundary,nIApp)
%
%       [IApp,CtrlFF,StimFF] = Func_FiringFrequencyCurve(Func,EXP,TH,tTransient,IAppBoundary,nIApp)
%
%   Function that permits to create the firing frequency bifurcation
%   diagram associated to the model passed as input in Func parameter once
%   stimulated with different experimental condition set up in EXP.
%
%   @param EXP          :   Experiment that needed to be reproduced.
%                           Its a structure containing the Control and 
%                           Stimulated fields. In each a param structure is
%                           contained denoted the parameter with which 
%                           execute the simulation.
%   @param TH           :   Threshold used to quantify if a spike is a 
%                           spike
%   @param tTransient   :   Transient time to wait before executing the
%                           firing frequency estimation.
%   @param IA[]Boundary :   Current boundary vector. First element lower
%                           boundary, second element upper boundary.
%   @param nIApp        :   Number of tested applied current for which
%                           execute the firing frequency estimation.
%
%   @output IApp        :   Current vector for which estimation made
%   @output CtrlFF      :   Control estimated Firing Frequency
%   @output StimFF      :   Stimulated estimated Firing Frequency
%
% PhD Student Martin Matteo
% University of Padova - Department of Information Engineering

% ========================== SETTINGS =====================================
options = odeset('RelTol',1e-6,'AbsTol',1e-6); 
CtrlFF = zeros(nIApp,1);  StimFF = zeros(nIApp,1);
TMP    = functions(Func); TMP    = TMP.function;
DIM    = str2double(regexp(TMP,'[0-9]','match'));
EQ     = zeros(DIM,1);
TSPAN  = [0 550];

% ======================== IAPP to TEST ===================================
IAppL = IAppBoundary(1); IAppH = IAppBoundary(2);
IApp  = linspace(IAppL,IAppH,nIApp);

% ========================== COMPUTATIONS =================================
figure(),
for i = 1:1:nIApp

    % ========================= CONTROL EXPERIMENT ========================
    EXP.Control.param.IApp = IApp(i);
    [t,q] = ode78(Func,TSPAN,EQ,options,EXP.Control.param);
    V = q(t>tTransient,1); V = V + abs(min(V)); tCtrl = t(t > tTransient); VCtrl = V; CtrlFF(i) = 0;
    if max(V) - min(V) > 10
        M = islocalmax(V); VPeaks = V(M); VPeaks = sort(VPeaks,'ascend');
        CtrlFF(i) = sum(VPeaks > VPeaks(end)*TH)/(TSPAN(2) - tTransient);
    end

    % ======================= STIMULATED EXPERIMENT =======================
    EXP.Stimulated.param.IApp = IApp(i);
    [t,q] = ode78(Func,TSPAN,EQ,options,EXP.Stimulated.param);
    V = q(t>tTransient,1); V = V + abs(min(V)); tStim = t(t > tTransient); VStim = V; StimFF(i) = 0;
    if max(V) - min(V) > 10
        M = islocalmax(V); VPeaks = V(M); VPeaks = sort(VPeaks,'ascend');
        StimFF(i) = sum(VPeaks > VPeaks(end)*TH)/(TSPAN(2) - tTransient);
    end

    % ====================== STATUS VISUALIZATION =========================
    subplot(2,1,1), plot(tCtrl, VCtrl, 'k'), 
                    ylabel(sprintf('CONTROL - Iteration %i',i)), 
                    xlabel(sprintf('Time [ms] - Frequency Esitimated f: %0.4f kHz',CtrlFF(i)))
    subplot(2,1,2), plot(tStim, VStim, 'k'), 
                    ylabel(sprintf('STIM - Iteration %i',i)), 
                    xlabel(sprintf('Time [ms] - Frequency Esitimated f: %0.4f kHz',StimFF(i)))
                    sgtitle(sprintf('Progress %2.2f%%',100*i/nIApp))
    drawnow
end
end