% 
% FIRING FREQUENCY
%
% ----- Note --------------------------------------------------------------
% This script implements the firing frequency analyses of the model. To do
% so, the script solves numerically at different applied current the system
% of ODEs.
%
% ----- Experiments -------------------------------------------------------
% In the following are listed the experiment used to execute the analyses.
%
%  EXPERIMENT      Control Condition   Stimulated Condition
%  EXP 1                  -                  CGS21680
%  EXP 2                ZD7288           ZD7288 + GCS21680
%  EXP 3               ZM241385        ZM241385 + CGS21680
%  EXP 4                  -                   XE991    
%  EXP 5                 FSK                FSK + Lin
%  EXP 6                XE991             XE991 + CGS2180                 
%
%
% ---- Requirements -------------------------------------------------------
% This script requires some folders with the down-below specified content.
%   -   Functions: Folder containing the functions to run the software 
%   -   Figures  : Folder containing the exported figure
%   -   Data     : Fodler where to store data
%
% ---- Author -------------------------------------------------------------
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

%% ENVIRONMENT
% Preparation of the environment.

clear all; close all; clc;

%% FUNCTIONS
% Functions folder.

addpath('Functions/')

%% FIGURE
% Name of the figure to be exported.

Fig1 = 'Figures/Fig_FF';

%% MODEL - PARAMETERS
% Definition of the standard and basic model parameters that will be used
% for further simulations of the mdoel behaviour.

param.Tmp   = 37    ; param.Cm   = 0.9 ;
param.gNaF  = 1000  ; param.gNaP = 1   ; 
param.gKS   = 40    ; param.gL   = 11.3;
param.DgHCN = 12    ; param.DgM  = 50  ;
param.gHCN  = 23    ; param.gM   = 50  ;
param.ENa   = 50    ; param.EK   = -84 ; 
param.ELK   = -83.38; param.EHCN = -50 ;
param.cAMP  = 0     ; param.IApp = 0   ;

param.Ps = 3.0^((param.Tmp - 20)/10);
param.Ph = 2.9^((param.Tmp - 20)/10); 
param.Pr = 3.0^((param.Tmp - 35)/10);

%% EXPERIMENTS - DEFINITIONS
% Definitions of the different performed experiments. These are coded as
% specific parameters combinations that are used to simulate the model and
% construct firing frequency diagrams.

nEXP = 6;

% ============================ EXPERIMENT 1 ===============================
% Administration of only CGS21680. This enhance the activation of cAMP
% dependent channels such as M and HCN ones. For this reason the only
% modification for the stimulated case consists in having cAMP parameter to
% 1.
EXP1.Control.param         = param;
EXP1.Stimulated.param      = EXP1.Control.param; 
EXP1.Stimulated.param.cAMP = 1;
EXP1.Control.legend        = '-';                
EXP1.Stimulated.legend     = 'CGS   ';

% ============================ EXPERIMENT 2 ===============================
% The second experiment administer always ZD7288, causing the inhibition of
% the HCN currents and then in the stimulated case it is administered the
% CGS21680 to enhance the M currents.
EXP2.Control.param         = param;
EXP2.Control.param.gHCN    = 0; 
EXP2.Control.param.DgHCN   = 0;
EXP2.Stimulated.param      = EXP2.Control.param; 
EXP2.Stimulated.param.cAMP = 1;
EXP2.Control.legend        = 'ZD';
EXP2.Stimulated.legend     = 'ZD/CGS';

% ============================ EXPERIMENT 3 ===============================
% Administration of ZM241385 in both situation by inhibiting the A2aR
% pathway.This lead to turn off both the alteration in the M and
% HCN channel conductances.
EXP3.Control.param         = param;              
EXP3.Control.param.DgHCN   = 12;
EXP3.Control.param.DgM     = 50;
EXP3.Stimulated.param      = EXP3.Control.param; 
EXP3.Stimulated.param.cAMP = 0;
EXP3.Control.legend        = 'ZM';
EXP3.Stimulated.legend     = 'ZM/CGS';

% ============================ EXPERIMENT 4 ===============================
% Administration of XE991 in the stimulated working condition. This lead to
% ihibit the M channel currents modelling this phenomenon through both the
% M channel conductances equal to zero.
EXP4.Control.param         = param;
EXP4.Stimulated.param      = EXP4.Control.param; 
EXP4.Stimulated.param.gM   = 0;
EXP4.Stimulated.param.DgM  = 0;
EXP4.Control.legend        = '-';
EXP4.Stimulated.legend     = 'XE    ';

% ============================ EXPERIMENT 5 ===============================
% Administration of pure Forskolin as a control condition and successive
% addition of XE causing the blockage of HCN channels.
EXP5.Control.param         = param;              
EXP5.Control.param.cAMP    = 1;
EXP5.Stimulated.param      = EXP5.Control.param; 
EXP5.Stimulated.param.gM   = 50;
EXP5.Stimulated.param.DgM  = 20;
EXP5.Control.legend        = 'FSK';             
EXP5.Stimulated.legend     = 'FSK/Lin';

% =========================== EXPERIMENT 6 ================================
% Administration of XE991 for the control condition and additional CGS21680
% for the stimulated case.
EXP6.Control.param         = param;
EXP6.Control.param.cAMP    = 0;
EXP6.Control.param.gM      = 0;
EXP6.Control.param.DgM     = 0;
EXP6.Stimulated.param      = EXP6.Control.param;
EXP6.Stimulated.param.cAMP = 1;
EXP6.Control.legend        = 'XE';
EXP6.Stimulated.legend     = 'XE/CGS';

%% EXPERIMENTS - ADMINISTRATION
% Reconstruction of the firing frequency curves. This is defined as the
% number of spikes in a given period are detected on average after a
% transient time in steady state. Before computing the firing frequency it
% is necessary to preelaborate the data avaialble by discriminating which
% oscillations can be considered as spikes. This happens through a
% threshold over the maximum oscillations amplitude.

IAppInterval = [0 700];
%IAppStep     = 0.01;
IAppStep     = 100;
nIAppTested  = (IAppInterval(2) - IAppInterval(1))/IAppStep;
tTransient   = 300;

ANS = input('USR - Do you want to compute new files? [0 - No|1 - Yes] ');
if ANS == 1
    fprintf('DEBUG - EXP1 - Starting...')
    [EXP1.IApp,EXP1.Control.FF,EXP1.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D, ...
        EXP1,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp1','EXP1')
    fprintf('Completed!\n')

    fprintf('DEBUG - EXP2 - Starting...')
    [EXP2.IApp,EXP2.Control.FF,EXP2.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D,...
        EXP2,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp2','EXP2')
    fprintf('Completed!\n')

    fprintf('DEBUG - EXP3 - Starting...')
    [EXP3.IApp,EXP3.Control.FF,EXP3.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D,...
        EXP3,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp3','EXP3')
    fprintf('Completed!\n')

    fprintf('DEBUG - EXP4 - Starting...')
    [EXP4.IApp,EXP4.Control.FF,EXP4.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D,...
        EXP4,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp4','EXP4')
    fprintf('Completed!\n')

    fprintf('DEBUG - EXP5 - Starting...')
    [EXP5.IApp,EXP5.Control.FF,EXP5.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D,...
        EXP5,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp5','EXP5')
    fprintf('Completed!\n')

    fprintf('DEBUG - EXP6 - Starting...')
    [EXP6.IApp,EXP6.Control.FF,EXP6.Stimulated.FF] = Func_FiringFrequencyCurve(@Func_LAV_5D,...
        EXP6,0.5,tTransient,IAppInterval,nIAppTested);
    fprintf('Operations Terminated ... Saving ... ')
    save('LAV_5D_FiringFrequency_Exp6','EXP6')
    fprintf('Completed!\n')
else
    load('Data/LAV_5D_FiringFrequency_Exp1.mat')
    load('Data/LAV_5D_FiringFrequency_Exp2.mat')
    load('Data/LAV_5D_FiringFrequency_Exp3.mat')
    load('Data/LAV_5D_FiringFrequency_Exp4.mat')
    load('Data/LAV_5D_FiringFrequency_Exp5.mat')
    load('Data/LAV_5D_FiringFrequency_Exp6.mat')
end

%% FIRING FREQUENCY - VISUALIZATION
% Visualization of the firing frequency analyses.

% ======================== EXPERIMENT: SELECTIONS =========================
% Selection of the experiment to be visualized inside the FF chart.

sEXP = [1, 6]; nEXP = length(sEXP);

% ========================= VISUALIZATION: COLORS =========================
% Definition of additional colors to be used to plot the different curves.

COL = [0,0,0;255,0,0;255,0,255;0,209,191]/255;

% ======================== VISUALIZATION: SETTINGS ========================
% Declaration of the visualization settings used to export the figure.

opts.width              = 10;                       % Figure Width  
opts.height             = 6;                        % Figure Height
opts.gridLineStyle      = '-.';                     % Grid
opts.minorGridLineStyle = '-.';                     % Minor Grid
opts.gridAlpha          = 0.05;                     % Grid Alpha
opts.minorGridAlpha     = 0.05;                     % Minor Grid Alpha
opts.FontSize           = 8;                        % Font Size
opts.FontName           = 'Times New Roman';        % Font Name
opts.FileExtension      = '.eps';                   % Extension of the file
opts.FileType           = '-depsc';                 % Type of file
opts.xlim               = [0 300];                  % X boundaries
opts.ylim               = [-0.005 0.7];             % Y boundaries

% ========================= VISUALIZATION: PLOT ===========================
% Visualization of the firing frequency curves.

fig = figure(); 
T   = tiledlayout(1,1,'TileSpacing','tight','Padding','compact');
nexttile(T);
for isEXP = 1:1:nEXP
    i = sEXP(isEXP);

    eval(['x = EXP',num2str(i),'.IApp;'])
    eval(['control = EXP',num2str(i),'.Control.FF;'])
    eval(['stimulated = EXP',num2str(i),'.Stimulated.FF;'])

    hold on,
    p((isEXP-1)*2+1) = plot(x, control,   'Color',COL((isEXP-1)*2+1,:),'LineWidth',1,'LineStyle','-');
    p((isEXP-1)*2+2) = plot(x, stimulated,'Color',COL((isEXP-1)*2+2,:),'LineWidth',1,'LineStyle','-');
    hold off
end

% ======================== VISUALIZATION: LEGEND ==========================
% Handling and visualization of the legend.

L = {'Control','CGS','XE','XE/CGS'};
L = legend(p,L,'NumColumns',4,'orientation','vertical');
L.ItemTokenSize = [15 15]'; 
L.FontName = opts.FontName; 
L.FontSize = opts.FontSize;
L.Layout.Tile = 'South';
legend box off; 

% ====================== VISUALIZATION: AXES ==============================
% Manual axes labels set up, and label visualization.

yLS = {'0.15','0.35','0.55'};  xLS = {'50','100','150','200','250'};
yLN = [0.15, 0.35, 0.55];      xLN = [50, 100, 150, 200, 250];
ylim(opts.ylim); yticks(yLN); yticklabels(yLS); 
xlim(opts.xlim); xticks(xLN); xticklabels(xLS); 
ylabel('$FF$ [$\mathrm{kHz}$]','interpreter','latex','FontSize',opts.FontSize,'FontName',opts.FontName);
xlabel('$I_{App}$ [$\mu \mathrm{A/cm^2}$]','interpreter','latex','FontSize',opts.FontSize,'FontName',opts.FontName);

% ============================ FIGURE: EXPORT =============================
% Export of the figure with the set up preferences.

Func_SaveFigure(gcf,opts,Fig1);