%
% SIMULATIONS
%
% ------ Note -------------------------------------------------------------
% Revised figure presenting various simulations of the 3D model. This
% script calculated the coefficients used to constrain linearly r on w.
% The code generates the figure presente in the paper.
%       
% ------ Requirements -----------------------------------------------------
% This file should be stored in a stand-alone folder containing some
% subfolders listed below.
%   - Function: Folder to store functions.
%   - Figures : Folder to store figures.
%   - Data    : Folder where to export data.
%
% ------ Author -----------------------------------------------------------
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova
%
% Last Rev - 20/11/2023

%% ENVIRONMENT
% Preparation of the enviroment.

clear all; close all; clc;

%% FUNCTIONS
% Functions used in the code.

addpath('Function/')

%% MODEL - VARIABLES
% Variables contained inside the model

VAR5D = {'V','h','s','r','w'};          % 5D Model
VAR3D = {'V','h','s'};                  % 3D Model

%% EXPERIMENTS - 5D MODEL
% Loading of the 5D model first experiments.

nEXP = 1; nIApp = 4;
load('Data/LAV_5D_Simulation_Exp1.mat');

%% AVERAGE VALUE
% Average value of the dynamical variables.

rAVGControl = [0.018065 0.016098]; rAVGStim = [0.054233 0.047454];
wAVGControl = [0.156478 0.168098]; wAVGStim = [0.094155 0.111086];

%% MODEL - r,w LINEAR CONSTRAINT
% Estimation of the parameter constraining onto a straight line the
% dynamical variable r and the dynamical variable w.

COL = ['k';'r'];
tTransient = 350;

for i = 1:1:nEXP
    for j = 1:1:nIApp
        % EXTRACTION
        eval(['IApp = EXP',num2str(i),'.Control.IApp(j);'])
        eval(['SC   = EXP',num2str(i),'.Control.S{',num2str(j),'};'])
        eval(['SS   = EXP',num2str(i),'.Stimulated.S{',num2str(j),'};'])
        eval(['LC   = EXP',num2str(i),'.Control.legend;'])
        eval(['LS   = EXP',num2str(i),'.Stimulated.legend;'])
        eval(['EXP  = EXP',num2str(i),';'])

        % CROPPING - Identification of instant of time to be visualized
        indexC = SC.t > tTransient; indexS = SS.t > tTransient;

        % GAIN COMPUTATION
        % CONTROL - Gain computation
        CKM   = Func_GainK(EXP.Control.param.cAMP, ...
                              EXP.Control.param.gM,EXP.Control.param.DgM);
        CKHCN = Func_GainK(EXP.Control.param.cAMP, ...
                              EXP.Control.param.gHCN,EXP.Control.param.DgHCN);
        % STIMUALTED - Gain computation
        SKM   = Func_GainK(EXP.Stimulated.param.cAMP, ...
                              EXP.Stimulated.param.gM,EXP.Stimulated.param.DgM);
        SKHCN = Func_GainK(EXP.Stimulated.param.cAMP, ...
                              EXP.Stimulated.param.gHCN,EXP.Stimulated.param.DgHCN);

        % LINEAR APPROXIMATION
        % CONTROL - Linear Approximation 
        G(:,1) = SC.w(indexC); G(:,2) = ones(sum(indexC),1); 
             Y = SC.r(indexC);      X = SC.w(indexC);
             P = inv(G'*G)*G'*Y;   
            PC = P*CKHCN;       PC(1) = PC(1)/CKM;
            XE = [0 1]';          YEC = XE*P(1) + P(2);
        clear G P Y X
        % STIMULATED - Linear Approximation 
        G(:,1) = SS.w(indexS); G(:,2) = ones(sum(indexS),1); 
             Y = SS.r(indexS);      X = SS.w(indexS);
             P = inv(G'*G)*G'*Y;
            PS = P*SKHCN;       PS(1) = PS(1)/SKM;
            XE = [0 1]';          YES = XE*P(1) + P(2);
        clear G P Y X

        % STORAGE
        eval(['EXP',num2str(i),'.Control.PE{j} = PC;'])
        eval(['EXP',num2str(i),'.Stimulated.PE{j} = PS;'])
    end
end

%% MODEL - VISUALIZATION
% Visualization of the affine line parameters m and q.

eval(['IApp = EXP',num2str(i),'.Control.IApp;'])
eval(['SC   = EXP',num2str(i),'.Control.PE;'])
eval(['SS   = EXP',num2str(i),'.Stimulated.PE;'])

for iI = 1:1:nIApp
    fprintf('EXP 1 - %i IApp - Control    m: %f\n',IApp(iI),SC{iI}(1))
    fprintf('EXP 1 - %i IApp - Control    q: %f\n',IApp(iI),SC{iI}(2))
    fprintf('EXP 1 - %i IApp - Stimulated m: %f\n',IApp(iI),SS{iI}(1))
    fprintf('EXP 1 - %i IApp - Stimulated q: %f\n',IApp(iI),SS{iI}(2))
end

%% MODEL - 3D SIMULATION
% Simulations of the 3D model.

ANS = input('Would you compute new simulations? [1 - Yes / 0 - No]');

if ANS == 1
    % SETTINGS
    eval(['EXP = EXP',num2str(i),';']), EXP.Control.S = {};
    IC = zeros(3,1); Options = odeset('RelTol',1e-6,'AbsTol',1e-6); TSPAN = [0 450];

    % CONTROL
    fprintf('DEBUG - Iteration: %i - CONTROL: ',i)
    EXP.Control.KM   = Func_GainK(EXP.Control.param.cAMP, ...
        EXP.Control.param.gM,EXP.Control.param.DgM);
    EXP.Control.KHCN = Func_GainK(EXP.Control.param.cAMP, ...
        EXP.Control.param.gHCN,EXP.Control.param.DgHCN);

    EXP.Control.param.r    = EXP.Control.EQ(4)*EXP.Control.KHCN;
    EXP.Control.param.w    = EXP.Control.EQ(5)*EXP.Control.KM;
    EXP.Control.param.IApp = 0;
    EQ = Func_FinalValue(@Func_LAV_3D,TSPAN,IC,VAR3D,...
        EXP.Control.param,Options);

    for j = 3:1:nIApp
        EXP.Control.param.IApp = EXP.Control.IApp(j);
        EXP.Control.param.r    = rAVGControl(j-2)*EXP.Control.KHCN;
        EXP.Control.param.w    = wAVGControl(j-2)*EXP.Control.KM;
        fprintf('- r: %f - w: %f ',EXP.Control.param.r,EXP.Control.param.w)
        EXP.Control.S{j} = Func_Simulate(@Func_LAV_3D,TSPAN,...
            EQ,VAR3D,EXP.Control.param,Options);
        fprintf('- %i/%i -',j-2,nIApp-2)
    end
    fprintf('COMPLETED!\n')

    % STIMULATED
    fprintf('DEBUG - Iteration: %i - STIMULATED: ',i)
    EXP.Stimulated.KM   = Func_GainK(EXP.Stimulated.param.cAMP, ...
        EXP.Stimulated.param.gM,EXP.Stimulated.param.DgM);
    EXP.Stimulated.KHCN = Func_GainK(EXP.Stimulated.param.cAMP, ...
        EXP.Stimulated.param.gHCN,EXP.Stimulated.param.DgHCN);

    EXP.Stimulated.param.r    = EXP.Stimulated.EQ(4)*EXP.Stimulated.KHCN;
    EXP.Stimulated.param.w    = EXP.Stimulated.EQ(5)*EXP.Stimulated.KM;
    EXP.Stimulated.param.IApp = 0;
    EQ = Func_FinalValue(@Func_LAV_3D,TSPAN,IC,VAR3D,...
        EXP.Stimulated.param,Options);

    for j = 3:1:nIApp
        EXP.Stimulated.param.IApp = EXP.Stimulated.IApp(j);
        EXP.Stimulated.param.r    = rAVGStim(j-2)*EXP.Stimulated.KHCN;
        EXP.Stimulated.param.w    = wAVGStim(j-2)*EXP.Stimulated.KM;
        fprintf('- r: %f - w: %f ',EXP.Stimulated.param.r,EXP.Stimulated.param.w)
        EXP.Stimulated.S{j} = Func_Simulate(@Func_LAV_3D,TSPAN,...
            EQ,VAR3D,EXP.Stimulated.param,Options);
        fprintf('- %i/%i -',j-2,nIApp-2)
    end
    fprintf('COMPLETED!\n')

    % EXPORT
    fprintf('DEBUG - Saving ... ')
    eval(['EXP',num2str(i),'.Control.S       = EXP.Control.S;'])
    eval(['EXP',num2str(i),'.Stimulated.S    = EXP.Stimulated.S;'])
    eval(['EXP',num2str(i),'.Control.KM      = EXP.Control.KM;'])
    eval(['EXP',num2str(i),'.Control.KHCN    = EXP.Control.KHCN;'])
    eval(['EXP',num2str(i),'.Stimulated.KM   = EXP.Stimulated.KM;'])
    eval(['EXP',num2str(i),'.Stimulated.KHCN = EXP.Stimulated.KHCN;'])
    eval(['save(''Data/LAV_3D_Simulation_Exp',num2str(i),''',''EXP',num2str(i),''');'])
    fprintf('Completed!\n\n')
else
    % LOADING
    load('Data/LAV_3D_Simulation_Exp1.mat')
end

%% VISUALIZATION
% Visualization of the figure presente in the paper.

% EXPERIMENT SELECITONS
iEXP = 1;
EXP_ToVisualize     = [false, false, true, true, false, false, true, true];
nTemporalEvolutions = sum(EXP_ToVisualize);

% SETTINGS
opts.gridLineStyle      = '-.';      
opts.minorGridLineStyle = '-.';
opts.gridAlpha          = 0.05;      
opts.minorGridAlpha     = 0.05;
opts.FontSize           = 8;         
opts.FontName           = 'Times New Roman';
opts.xlim               = [325 375]; 
opts.ylim               = [-90 30];
opts.FileExtension      = '.eps'; 
opts.FileType           = '-depsc';
%opts.width              = 12; 
opts.width              = 11.4; 
%opts.height             = 4.613;
opts.height             = 4.5;

% FILE NAME
fileImage = 'Figures/Fig_Sim1_3D'; 

% AXIS LABEL
[yLS,yLN] = Func_AxisLabel(3,opts.ylim,0);
[xLS,xLN] = Func_AxisLabel(3,opts.xlim,0);
xLS{1} = '335'; xLN(1) = 335; 
xLS{3} = '365'; xLN(3) = 365;

% FIGURE
fig = figure();
T = tiledlayout(1,2,'Padding','Tight','TileSpacing','Tight');
xlabel(T,'$t$ [ms]','interpreter','latex','FontName','Times New Roman','FontSize',8);
title(T,'$I_{App}$','interpreter','latex','FontName','Times New Roman','FontSize',8);


% CONTROL
for iTE = 1:1:nTemporalEvolutions
    if EXP_ToVisualize(iTE)
        TC = tiledlayout(T,1,nTemporalEvolutions/2,'Padding','Tight','TileSpacing','Tight');
        TC.Layout.Tile = 1;

        eval(['SC   = EXP',num2str(iEXP), '.Control.S{',num2str(iTE),'};'])
        eval(['IApp = EXP',num2str(iEXP), '.Stimulated.IApp(',num2str(iTE),');'])

        nexttile(TC, iTE-2),
        plot(SC.t, SC.V, 'Color','k','LineWidth',1);

        ylim(opts.ylim), xlim(opts.xlim)
        axes = gca; set(axes,'xtick',xLN); set(axes,'xticklabel',xLS);
        axes = gca; set(axes,'ytick',yLN); set(axes,'yticklabel',{});

        if iTE == 3
            axes = gca; set(axes,'ytick',yLN); set(axes,'yticklabel',yLS);
            ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8);
        end

        title([num2str(IApp),' [$\mu$A/cm$^2$]'],'interpreter','latex','FontName','Times New Roman','FontSize',8);
    end
end
xlabel(TC,'Control','interpreter','latex','FontName','Times New Roman','FontSize',8);

% STIMUALTED
for iTE = 1:1:nTemporalEvolutions
    if EXP_ToVisualize(iTE+4)
        TS = tiledlayout(T,1,nTemporalEvolutions/2,'Padding','Tight','TileSpacing','Tight');
        TS.Layout.Tile = 2;

        eval(['SS   = EXP',  num2str(iEXP), '.Stimulated.S{',num2str(iTE),'};'])
        eval(['LS   = EXP',  num2str(iEXP), '.Stimulated.legend;'])
        eval(['IApp = EXP',num2str(iEXP), '.Stimulated.IApp(',num2str(iTE),');'])

        nexttile(TS, iTE-2),
        plot(SS.t, SS.V, 'Color','r','LineWidth',1);

        ylim(opts.ylim), xlim(opts.xlim)
        axes = gca; set(axes,'xtick',xLN); set(axes,'xticklabel',xLS);
        axes = gca; set(axes,'ytick',yLN); set(axes,'yticklabel',{});

        if iTE == 4
            axes = gca; set(axes,'ytick',yLN); set(axes,'yticklabel',yLS);
            ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8);
            set(axes,'yaxislocation','right');
        end

        title([num2str(IApp),' [$\mu$A/cm$^2$]'],'interpreter','latex','FontName','Times New Roman','FontSize',8);
    end
end
xlabel(TS,'CGS','interpreter','latex','FontName','Times New Roman','FontSize',8);

% EXPORT
Func_SaveFigure(fig,opts,sprintf(fileImage,iEXP));

