%
% SIMULATIONS
%
% ------ Note -------------------------------------------------------------
% Revised figure presenting various simulations of the model, under
% different pharmacological and external current stimulations.
% The first time you run the algorithm make sure to compute new
% simulations. If you desire to run again the algorithm then you can skip
% the numerical solutions of the ODEs system. In this case, the script load
% the previously computed solutions.
%       
% ------ Requirements -----------------------------------------------------
% This file should be stored in a stand-alone folder containing some
% subfolders listed below.
%   - Function: Folder to store functions.
%   - Figure  : Folder to store figures.
%   - Data    : Folder where to export data.
%
% ------ Author -----------------------------------------------------------
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova
%
% Last Rev - 20/11/2023

%% ENVIRONMENT
% Enviroment preparation.

clear all; clc;

%% FUNCTIONS
% Functions used to handle the code.

addpath('./Function/')

%% MODEL - PARAMETERS
% Default parameters.

param.Tmp   = 37    ; param.Cm   = 0.9 ;
param.gNaF  = 1000  ; param.gNaP = 1   ; 
param.gKS   = 40    ; param.gL   = 11.3;
param.DgHCN = 12    ; param.DgM  = 50  ;
param.gM    = 50    ; param.gHCN = 23  ;
param.ENa   = 50    ; param.EK   = -84 ; 
param.ELK   = -83.38; param.EHCN = -50 ;
param.cAMP  = 0     ; param.IApp = 0   ;

param.Pm = 2.2^((param.Tmp - 20)/10); 
param.Ps = 3.0^((param.Tmp - 20)/10);
param.Ph = 2.9^((param.Tmp - 20)/10); 
param.Pn = 2.2^((param.Tmp - 20)/10);
param.Pr = 3.0^((param.Tmp - 35)/10);

%% FIGURES
% Name of the figures to be exported.

Fig1 = 'Figure/Fig_Sim1';
Fig2 = 'Figure/Fig_Sim2';
Fig3 = 'Figure/Fig_Sim3';

%% IAPP
% Vector specifying the external stimulating current.

IAppV = [115, 200, 250, 300];

%% MODEL - VARIABLES
% Variables

VAR5D = {'V','h','s','r','w'};

%% EXPERIMENTS - DEFINITIONS
% Definitions of the different performed experiments. These are coded as
% specific parameters combinations that are used to simulate the model and
% construct firing frequency diagrams.

nEXP = 6; nIApp = length(IAppV);

% ============================ EXPERIMENT 1 ===============================
% Control -> None                                    Stimulated -> CGS21680

% Control
EXP1.Control.param         = param;
EXP1.Control.legend        = 'none';
EXP1.Control.IApp          = IAppV;

% CGS21680
EXP1.Stimulated.param      = EXP1.Control.param; 
EXP1.Stimulated.param.cAMP = 1;
EXP1.Stimulated.legend     = 'CGS';
EXP1.Stimulated.IApp       = IAppV;

% ============================ EXPERIMENT 2 ===============================
% Control -> ZD7288                         Stimulated -> ZD7288 + CGS21680

% ZD7288
EXP2.Control.param         = param;              
EXP2.Control.param.gHCN    = 0; 
EXP2.Control.param.DgHCN   = 0;
EXP2.Control.legend        = 'ZD';
EXP2.Control.IApp          = IAppV;

% ZD7288/CGS21680
EXP2.Stimulated.param      = EXP2.Control.param; 
EXP2.Stimulated.param.cAMP = 1;
EXP2.Stimulated.legend     = 'ZD/CGS';
EXP2.Stimulated.IApp       = IAppV;

% ============================ EXPERIMENT 3 ===============================
% Control -> ZD241385                     Stimulated -> ZM241385 + CGS21680

% ZM241385
EXP3.Control.param         = param;
EXP3.Control.param.DgHCN   = 12;
EXP3.Control.param.DgM     = 50;
EXP3.Control.legend        = 'ZM';
EXP3.Control.IApp          = IAppV;

% ZM241385/CGS21680
EXP3.Stimulated.param      = EXP3.Control.param; 
EXP3.Stimulated.param.cAMP = 0; 
EXP3.Stimulated.legend     = 'ZM/CGS';
EXP3.Stimulated.IApp       = IAppV;

% ============================ EXPERIMENT 4 ===============================
% Control -> None                                       Stimulated -> XE991

% Control
EXP4.Control.param        = param;
EXP4.Control.legend       = 'none';
EXP4.Control.IApp         = IAppV ;

% XE991
EXP4.Stimulated.param     = EXP4.Control.param;
EXP4.Stimulated.param.gM  = 0;
EXP4.Stimulated.param.DgM = 0;
EXP4.Stimulated.legend    = 'XE';
EXP4.Stimulated.IApp      = IAppV;

% ============================ EXPERIMENT 5 ===============================
% Control -> Forskolin                Stimulated -> Forskolin + Linopirdine

% Forskolin
EXP5.Control.param        = param;
EXP5.Control.param.cAMP   = 1;
EXP5.Control.legend       = 'FSK';
EXP5.Control.IApp         = IAppV;

% Forskolin/Linopirdine
EXP5.Stimulated.param     = EXP5.Control.param;
EXP5.Stimulated.param.DgM = 20;
EXP5.Stimulated.legend    = 'FSK/Lin';
EXP5.Stimulated.IApp      = IAppV;

% ============================ EXPERIMENT 6 ===============================
% Control -> XE991                           Stimulated -> XE991 + CGS21680

% XE991
EXP6.Control.param     = param;
EXP6.Control.param.gM  = 0; 
EXP6.Control.param.DgM = 0;
EXP6.Control.legend    = 'XE';  
EXP6.Control.IApp      = IAppV; 

% XE991/CGS21680
EXP6.Stimulated.param      = EXP6.Control.param;
EXP6.Stimulated.param.cAMP = 1;
EXP6.Stimulated.legend     = 'XE/CGS';
EXP6.Stimulated.IApp       = IAppV;

%% SIMULATION - COMPUTATION
% Simulation of the model using standard ode solvers.

ANS = input('USR - Do you want to compute new simulations? [1 - Yes|0 - No] ');

if ANS == 1
    for i = 1:1:nEXP
        % ============================= SETTINGS ==========================
        eval(['EXP = EXP',num2str(i),';']) 
        IC    = zeros(5,1); 
        nIApp = length(EXP.Control.IApp);
        opts  = odeset('RelTol',1e-6,'AbsTol',1e-6); 
        TSPAN = [0 450];
    
        fprintf('DEBUG - Iteration: %i - CONTROL: ',i)
        % ============================ CONTROL ============================
        EXP.Control.param.IApp = 0;
        % EQUILIBRIUM
        EQC = Func_FinalValue(@Func_LAV_5D,TSPAN,IC,VAR5D,EXP.Control.param,opts);
        fprintf('EQ Reached - Simulating ')
        % STIMULATION
        for j = 1:1:nIApp
            EXP.Control.param.IApp = EXP.Control.IApp(j);
            EXP.Control.S{j} = Func_Simulate(@Func_LAV_5D,TSPAN,EQC,VAR5D,EXP.Control.param,opts);
            fprintf('- %i/%i -',j,nIApp)
        end
        fprintf('COMPLETED!\n')
    
        fprintf('DEBUG - Experiment: %i - STIMULATED: ',i)
        % ============================ STIMULATED =========================
        EXP.Stimulated.param.IApp = 0;
        % EQUILIBRIUM
        EQS = Func_FinalValue(@Func_LAV_5D,TSPAN,IC,VAR5D,EXP.Stimulated.param,opts);
        fprintf('EQ Reached - Simulating ')
        % STIMULATION
        for j = 1:1:nIApp
            EXP.Stimulated.param.IApp = EXP.Stimulated.IApp(j);
            EXP.Stimulated.S{j} = Func_Simulate(@Func_LAV_5D,TSPAN,EQS,VAR5D,EXP.Stimulated.param,opts);
            fprintf('- %i/%i -',j,nIApp)
        end
        fprintf('COMPLETED!\n')
    
        fprintf('DEBUG - Saving ... ')
        % ============================ SAVING =============================
        eval(['EXP',num2str(i),'.Control.S = EXP.Control.S;'])
        eval(['EXP',num2str(i),'.Stimulated.S = EXP.Stimulated.S;'])
        eval(['EXP',num2str(i),'.Control.EQ = EQC;'])
        eval(['EXP',num2str(i),'.Stimulated.EQ = EQS;'])
        eval(['save(''Data/LAV_5D_Simulation_Exp',num2str(i),''',''EXP',num2str(i),''');'])
        fprintf('Completed!\n\n')
    end

    clear opts
else
    load('Data/LAV_5D_Simulation_Exp1.mat')
    load('Data/LAV_5D_Simulation_Exp2.mat')
    load('Data/LAV_5D_Simulation_Exp3.mat')
    load('Data/LAV_5D_Simulation_Exp4.mat')
    load('Data/LAV_5D_Simulation_Exp5.mat')
    load('Data/LAV_5D_Simulation_Exp6.mat')
end

%% SIMULATIONS - VISUALIZATIONS (1)
% Visualization of the model simulations.

% ========================== FIGURE: SETTINGS =============================
% Settings used to handle the figure.

opts.width              = 12;                % Figure Width
opts.height             = 6.6;               % Figure Height
opts.gridLineStyle      = '-.';              % Grid Style
opts.minorGridLineStyle = '-.';              % Minor Grid Style
opts.gridAlpha          = 0.05;              % Grid Alpha
opts.minorGridAlpha     = 0.05;              % Minor Grid Alpha
opts.FontSize           = 8;                 % Font Size
opts.FontName           = 'Times New Roman'; % Font Name
opts.xlim               = [350 400];         % X Boundaries
opts.ylim               = [-90 30];          % Y Boundaries
opts.FileExtension      = '.eps';            % Extension of File
opts.FileType           = '-depsc';          % Type of File

% ======================== FIGURE: LAYOUT =================================
% The figure is organized in two lines, the first presents the control
% condition instead the second the CGS stimulated case.

fig = figure();

TL  = tiledlayout(2,1,'TileSpacing','tight','Padding','compact');
title(TL,'$I_{App}$','interpreter','latex','FontSize',8)
xlabel(TL,'$t$ [ms]','interpreter','latex','FontName','Times New Roman','FontSize',8)

TL1 = tiledlayout(TL,1,4,'TileSpacing','tight','Padding','tight');
TL1.Layout.Tile = 1; ylabel(TL1,'Control','FontName','Times New Roman','FontSize',8);

TL2 = tiledlayout(TL,1,4,'TileSpacing','tight','Padding','tight');
TL2.Layout.Tile = 2; ylabel(TL2,'CGS','FontName','Times New Roman','FontSize',8);

% ===================== FIGURE: AXES LABELS ===============================
% The axes labels are manually set to visualize some "nice" numbers.

xLN = [360, 375, 390];  xLS = {'360','375','390'};
yLN = [-60, -30,   0];  yLS = {'-60','-30','0'};

% =============================== EXPERIMENT 1 ============================
for iSIM = 1:1:length(EXP1.Control.IApp)
    IApp = EXP1.Control.IApp(iSIM);

    % Control
    nexttile(TL1,iSIM)
    plot(EXP1.Control.S{iSIM}.t,EXP1.Control.S{iSIM}.V,'Color','k','LineWidth',1)

    title(sprintf('%i [$\\mu$A/cm$^2$]',IApp),'interpreter','latex')
    xlim(opts.xlim), ylim(opts.ylim)
    xticklabels({}), xticks(xLN)
    
    if iSIM == 1 || iSIM == 4
        yticklabels(yLS); yticks(yLN);
        ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8)
        if iSIM == 4, set(gca,'YAxisLocation','right'); end
    else
        yticklabels({}) ; yticks(yLN);
    end

    % CGS
    nexttile(TL2,iSIM)
    plot(EXP1.Stimulated.S{iSIM}.t,EXP1.Stimulated.S{iSIM}.V,'Color','r','LineWidth',1)

    xlim(opts.xlim) , ylim(opts.ylim)
    xticklabels(xLS), xticks(xLN)
    
    if iSIM == 1 || iSIM == 4
        yticklabels(yLS); yticks(yLN);
        ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8);
        if iSIM == 4, set(gca,'YAxisLocation','right'); end
    else
        yticklabels({}) ; yticks(yLN);
    end
end

% ============================== FIGURE: EXPORT ===========================
% Export of the figure using the previously specified settings.
Func_SaveFigure(fig,opts,Fig1);

%% SIMULATIONS - VISUALIZATIONS (2)
% Visualization of the model simulations.

% ========================== FIGURE: SETTINGS =============================
% Settings used to handle the figure.

opts.width              = 13.2;              % Figure Width
opts.height             = 7.1;               % Figure Height
opts.gridLineStyle      = '-.';              % Grid Style
opts.minorGridLineStyle = '-.';              % Minor Grid Style
opts.gridAlpha          = 0.05;              % Grid Alpha
opts.minorGridAlpha     = 0.05;              % Minor Grid Alpha
opts.FontSize           = 8;                 % Font Size
opts.FontName           = 'Times New Roman'; % Font Name
opts.xlim               = [350 400];         % X Boundaries
opts.ylim               = [-90 30];          % Y Boundaries
opts.FileExtension      = '.eps';            % Extension of File
opts.FileType           = '-depsc';          % Type of File

% ============================ FIGURE: COLORS =============================
% Implementation of the colors used to distinguish the different cases.
c.m = [255,   0, 255]/255; c.t = [0  , 209, 191]/255; 
c.a = [255, 149,   5]/255; c.v = [120,   0, 255]/255;

% ======================== FIGURE: LAYOUT =================================
% The figure is organized in four cells organized in two lines and two 
% columns, each containing two plots. Each cell depicts simulations of the
% model under different pharmacological stimulation.

fig = figure(); 
%TL  = tiledlayout(2,1,'TileSpacing','compact','Padding','compact');
TL  = tiledlayout(1,2,'TileSpacing','compact','Padding','compact');
title(TL,'$I_{App}$','interpreter','latex','FontSize',8)
xlabel(TL,'$t$ [ms]','interpreter','latex','FontName','Times New Roman','FontSize',8)

%TL1 = tiledlayout(TL,1,2,'TileSpacing','tight','Padding','tight');
TL1 = tiledlayout(TL,2,2,'TileSpacing','tight','Padding','tight');
TL1.Layout.Tile = 1; %ylabel(TL1,'Control','FontName','Times New Roman','FontSize',8);
xlabel(TL1,'ZD','FontName','Times New Roman','FontSize',8);

%TL1C1 = tiledlayout(TL1,1,2,'TileSpacing','tight','Padding','tight');
%TL1C1.Layout.Tile = 1;

%TL1C2 = tiledlayout(TL1,1,2,'TileSpacing','tight','Padding','tight');
%TL1C2.Layout.Tile = 2;

%TL2 = tiledlayout(TL,1,2,'TileSpacing','tight','Padding','tight');
TL2 = tiledlayout(TL,2,2,'TileSpacing','tight','Padding','tight');
TL2.Layout.Tile = 2; %ylabel(TL2,'CGS','FontName','Times New Roman','FontSize',8);


%TL2C1 = tiledlayout(TL2,1,2,'TileSpacing','tight','Padding','tight');
%TL2C1.Layout.Tile = 1; xlabel(TL2C1,'ZD','FontName','Times New Roman','FontSize',8);

%TL2C2 = tiledlayout(TL2,1,2,'TileSpacing','tight','Padding','tight');
%TL2C2.Layout.Tile = 2; xlabel(TL2C2,'XE','FontName','Times New Roman','FontSize',8);
xlabel(TL2,'XE','FontName','Times New Roman','FontSize',8);

% ===================== FIGURE: AXES LABELS ===============================
% The axes labels are manually set to visualize some "nice" numbers. The
% first cell of plots has some special labels.

xLN  = [360, 375, 390];     xLNS = [325, 375, 425];     yLN  = [-60,-30,0];
xLS  = {'360','375','390'}; xLSS = {'325','375','425'}; yLS  = {'-60','-30','0'};

T    = [1,nan,nan,2];
jSIM = 1;

% =============================== EXPERIMENT 2 ============================
% The first two cells visualize the experiment 2 simulations. Only the
% simulations associated with IApp equal to 115 and 300 are visualized.
for iSIM = 1:1:length(EXP2.Control.IApp)
    IApp = EXP2.Control.IApp(iSIM);
    if isnan(T(iSIM)), continue; end

    % ZD
    %nexttile(TL1C1)
    nexttile(TL1,jSIM)
    plot(EXP2.Control.S{iSIM}.t,EXP2.Control.S{iSIM}.V,'Color',c.a,'LineWidth',1)
    
    title(sprintf('%i [$\\mu$A/cm$^2$]',IApp),'interpreter','latex')
    ylim(opts.ylim), xticklabels({})
    
    if iSIM ~= 1
        xlim(opts.xlim);  xticks(xLN);
        yticklabels({}); yticks(yLN);
    else
        ylabel({'No CGS','$V$ [mV]'},'interpreter','latex','FontName','Times New Roman','FontSize',8)
        xlim([300 450]);  xticks(xLNS);
        yticklabels(yLS) ; yticks(yLN);
    end

    % ZD/CGS
    %nexttile(TL2C1)
    nexttile(TL1,2+jSIM)
    plot(EXP2.Stimulated.S{iSIM}.t,EXP2.Stimulated.S{iSIM}.V,'Color',c.v,'LineWidth',1)

    ylim(opts.ylim)
    
    if iSIM ~= 1
        xlim(opts.xlim), xticks(xLN), xticklabels(xLS)
        yticklabels({}); yticks(yLN);
    else
        ylabel({'CGS','$V$ [mV]'},'interpreter','latex','FontName','Times New Roman','FontSize',8)
        xlim([300 450]), xticks(xLNS), xticklabels(xLSS)
        yticklabels(yLS) ; yticks(yLN);
    end

    jSIM = jSIM + 1;
end

T = [3,nan,nan,4]; jSIM = 1;

% =============================== EXPERIMENT 6 ============================
% The first two cells visualize the experiment 6 simulations. Only the
% simulations associated with IApp equal to 115 and 300 are visualized.
for iSIM = 1:1:length(EXP6.Control.IApp)
    IApp = EXP6.Control.IApp(iSIM);
    if isnan(T(iSIM)), continue; end

    % XE
    %nexttile(TL1C2)
    nexttile(TL2,jSIM)
    plot(EXP6.Control.S{iSIM}.t,EXP6.Control.S{iSIM}.V,'Color',c.m,'LineWidth',1)
    
    title(sprintf('%i [$\\mu$A/cm$^2$]',IApp),'interpreter','latex')
    xlim(opts.xlim) , ylim(opts.ylim)
    xticklabels({}) , xticks(xLN)

    if iSIM == 4
        yticklabels(yLS); yticks(yLN);
        set(gca,'YAxisLocation','right');
        ylabel({'$V$ [mV]','No CGS'},'interpreter','latex','FontName','Times New Roman','FontSize',8)
    else
        yticklabels({}) ; yticks(yLN);
    end

    % XE/CGS
    %nexttile(TL2C2)
    nexttile(TL2,2+jSIM)
    plot(EXP6.Stimulated.S{iSIM}.t,EXP6.Stimulated.S{iSIM}.V,'Color',c.t,'LineWidth',1)

    xlim(opts.xlim) , ylim(opts.ylim)
    xticklabels(xLS), xticks(xLN)

    if iSIM == 4,  end
    if iSIM == 4
        yticklabels(yLS); yticks(yLN);
        set(gca,'YAxisLocation','right');
        ylabel({'$V$ [mV]','CGS'},'interpreter','latex','FontName','Times New Roman','FontSize',8)
    else
        yticklabels({}) ; yticks(yLN);
    end

    jSIM = jSIM + 1;
end

% ============================== FIGURE: EXPORT ===========================
% Export of the figure using the previously specified settings.
Func_SaveFigure(fig,opts,Fig2);
%Func_SaveFigure(fig,opts,'');

%% SIMULATIONS - VISUALIZATIONS (3)
% Visualization of the model simulations.

% ========================== FIGURE: SETTINGS =============================
% Settings used to handle the figure.

%opts.width              = 6.78;              % Figure Width
opts.width              = 6.8;              % Figure Width
%opts.height             = 4.613;             % Figure Height
opts.height             = 4.7;             % Figure Height
opts.gridLineStyle      = '-.';              % Grid Style
opts.minorGridLineStyle = '-.';              % Minor Grid Style
opts.gridAlpha          = 0.05;              % Grid Alpha
opts.minorGridAlpha     = 0.05;              % Minor Grid Alpha
opts.FontSize           = 8;                 % Font Size
opts.FontName           = 'Times New Roman'; % Font Name
opts.xlim               = [350 400];         % X Boundaries
opts.ylim               = [-90 30];          % Y Boundaries
opts.FileExtension      = '.eps';            % Extension of File
opts.FileType           = '-depsc';          % Type of File

% ============================ FIGURE: COLORS =============================
% Implementation of the colors used to distinguish the different cases.

c.b = [0,  0,255]/255;  c.g = [120,120,120]/255;

% ======================== FIGURE: LAYOUT =================================
% The figure is organized in four cells organized in two lines and two 
% columns, each containing two plots. Each cell depicts simulations of the
% model under different pharmacological stimulation.

fig = figure(); 
TL  = tiledlayout(1,2,'TileSpacing','tight','Padding','compact');
title(TL,'$I_{App}$','interpreter','latex','FontSize',8)
xlabel(TL,'$t$ [ms]','interpreter','latex','FontName','Times New Roman','FontSize',8)

% ===================== FIGURE: AXES LABELS ===============================
% The axes labels are manually set to visualize some "nice" numbers.

xLN  = [360, 375, 390];     yLN  = [-60,-30,0];
xLS  = {'360','375','390'}; yLS  = {'-60','-30','0'};

% =============================== EXPERIMENT 5 ============================
iSIM = 2;
IApp = EXP5.Control.IApp(2);

% FSK
nexttile(TL)
plot(EXP5.Control.S{iSIM}.t,EXP5.Control.S{iSIM}.V,'Color',c.b,'LineWidth',1)

xlabel('FSK','FontSize',8,'FontName','Times New Roman')
ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8)
title(sprintf('%i [$\\mu$A/cm$^2$]',IApp),'interpreter','latex')
ylim(opts.ylim);  xlim(opts.xlim);
xticklabels(xLS); xticks(xLN);
yticklabels(yLS); yticks(yLN);

% FSK/Lin
nexttile(TL)
plot(EXP5.Stimulated.S{iSIM}.t,EXP5.Stimulated.S{iSIM}.V,'Color',c.g,'LineWidth',1)

set(gca,'YAxisLocation','right')
xlabel('FSK/Lin','FontSize',8,'FontName','Times New Roman')
ylabel('$V$ [mV]','interpreter','latex','FontName','Times New Roman','FontSize',8)
title(sprintf('%i [$\\mu$A/cm$^2$]',IApp),'interpreter','latex')
xlim(opts.xlim);  ylim(opts.ylim);
xticklabels(xLS); xticks(xLN);
yticklabels(yLS); yticks(yLN);

% ============================== FIGURE: EXPORT ===========================
% Export of the figure using the previously specified settings.
Func_SaveFigure(fig,opts,Fig3);
