function S = Func_CanardOrbitApproximation(SigmaSet, param, CCLB, CCLF, Geom, Event, LAOTH, isVisualProgress)
%
%   S = Func_CanardOrbitApproximation(SigmaSet, param, CCLB, CCLF,...
%                                   Geom, Event, LAOTH, isVisualProgress)
%
%   Function that approximate each canard orbit by optimizing the
%   termination point between forward and backward dynamical system
%   solution. The output structure contains a number of orbits equal to the
%   number of plane that are specified inside the input of this function.
%   In particular, depending on the number of planes specified inside
%   SigmaSet a given number of orbits is computed too.
%
%   @param SigmaSet     :   Set of planes where to find the approximated
%                           canard orbit.
%   @param param        :   Parameter structure with which the model is
%                           simulated and the analysis performed.
%   @param CCLB         :   Initial condition from which start the backward
%                           integration of the solution.
%   @param CCLF         :   Initial condition from which start the forward
%                           integration of the system.
%   @param Geom         :   Geometry of the model.
%   @param Event        :   Event function use to end the integration.
%   @param LAOTH        :   Numerical value associated to the threshold
%                           used to discriminate between large and small
%                           amplitude oscillations.
%   @param isVisualPro. :   Flag used to check the progress of the
%                           optimization algorithm.
%
%   @output S           :   Structure of candidate orbit.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering


% ============================ INITIALIZATION =============================
% Initialization of certain variables used to infer information based on
% the number of optimization problem to be solved and the initialization of
% a variable collection that will contain the whole pull of approximated
% solutions.
nS = length(SigmaSet); S = {};

for iS = 1:1:nS

    % ========================= MARKER: CODE STATUS =======================
    % Marker use to track the code status.
    fprintf('OPTIMIZING - Process %i/%i -',iS,nS)
    
    % ========================== PLANE: SELECTION =========================
    % Selection of the plane considered for integration-termination.
    % plane
    param.Sigma = SigmaSet{iS};

    % ========================== ORBIT: APPROXIMATION =====================
    % Approximation of the canard orbit via the solution of the
    % optimization problem previously specified.
    [S{iS}.S,S{iS}.e] = Func_OptimizeICSolution(Geom, param, CCLB, CCLF, isVisualProgress, Event);

    % =========================== INFORMATION: RETRIVAL ===================
    % Retrival of some information needed to print out the main feature of
    % the considered and obtained approximated solution.
    iPeaks = islocalmax(S{iS}.S.V); iPeaks(S{iS}.S.i) = false;
    peaks  = S{iS}.S.V(iPeaks);     S{iS}.nSAO = sum(peaks < LAOTH);
    
    % ========================= MARKER: CODE STATUS =======================
    % Marker use to track the code status.
    fprintf('SAO: %i - Error: %f\n',S{iS}.nSAO, S{iS}.e)
end

% =========================== MARKER: CODE STATUS =========================
% Marker use to track the code status.
fprintf('OPTIMIZING - Completed!\n')


end