function [SCF, SCB, Error] = Func_ComputeCanard(ICF,ICB,FuncSys,param,VAR3D,P,IF,IB,IP,Visualize,E)
%
%   [SCF, SCB, Error] = Func_ComputeCanard(ICF,ICB,FuncSys,param,VAR3D,P,IF,...
%                               IB,IP,Visualize,E)
%
%   Function that enable the computation of the canard solution once
%   passed as input to the function information based on the estimated
%   initial conditions over the line where both forward and backward
%   integration has been executed.
%   
%   @param ICF      :   Initial condition canard forward integration
%   @param ICB      :   Initial condition canard backward integration
%   @param FuncSys  :   Function descriptive of the 3D system to simulate
%   @param param    :   Structure containing all the parameters necessary
%                       to execute the simulation
%   @param VAR3D    :   Structure which contains the variable order that 
%                       should be visualized.
%   @param P        :   Vector representative of the center point respect
%                       which the plane projection are calculated.
%   @param IF       :   Projection of the forward simulation of the 
%                       dynamical system.
%   @param IB       :   Projection of the backward simulation of the 
%                       dynamical system.
%   @param IP       :   Intersection point between the curves projected
%                       over the plane estimated with polyxpoly
%   @param Visualize:   Boolean value that turn on and turn off the
%                       visualization.
%   @param E        :   Event for which compute the intersection between
%                       the canards identified and the plane.
%
%   @output SCF     :   Structure of the forward simulated canard orbits
%   @output SCB     :   Structure of the backward simulated canard orbits
%   @output Error   :   Projection error structure between the forward,
%                       backward simulations and polyxpoly algorithm of the
%                       canard orbits.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% ============================ INFORMATIONS  ==============================
% General purposes information retrived to execute the code correctly.
nC = size(ICF,1);

% =============================== CANARD ==================================
% Computation of the canard orbits via forward and backward solution of
% simil- BVP from which the initial condition is constrained to belong in a
% given line in space and the trajectory end point inside the Sigma plane.
param.isT = zeros(2,1); param.isT(E) = 1;
param.TL = 0; param.TH = 100;  SCF = Func_BVP(ICF,FuncSys,param,0,0,VAR3D);
param.TL = 0; param.TH = -100; SCB = Func_BVP(ICB,FuncSys,param,0,0,VAR3D);

% ============================== INTERSECTIONS ============================
% Identification of the plane intersection of the previously obtained
% canard solution to check the reliability inside the last point projected
% on the phase plane on the target plane.
if E == 1, Plane = param.SigmaFN; end
if E == 2, Plane = param.SigmaDW; end
IFN = Func_ComputePlaneIntersection(Plane,SCF,P,E,Visualize);
IBN = Func_ComputePlaneIntersection(Plane,SCB,P,E,Visualize);

% ================================ ERRORS =================================
% Error quantification between the intersection estimated via the polyxpoly
% algorithm and the forward/backward simulations projected over the plane
% Sigma.
for i = 1:1:size(ICF,1)
    PF = [IFN{i}.x1; IFN{i}.x2];
    PB = [IBN{i}.x1; IBN{i}.x2];
    PI = [IP{i}.x1; IP{i}.x2];
    Error.BF(i) = norm(PF - PB);
    Error.BI(i) = norm(PB - PI);
    Error.FI(i) = norm(PF - PI);
end

% ============================ VISUALIZATION ==============================
% Visualization of forward, backward, canard forward, canard backward 
% projections and interesection points between the curves considered.
% Visualization of the discrepancy between the estimated forward and
% backward solutions.
if Visualize
    figure, Func_VisualizePlaneProjection(IF,{'x1','x2'},'b');
            Func_VisualizePlaneProjection(IB,{'x1','x2'},'r'); 
            Func_VisualizePlaneProjection(IFN,{'x1','x2'},'m');
            Func_VisualizePlaneProjection(IBN,{'x1','x2'},'g'); 
            Func_VisualizePlaneProjection(IP,{'x1','x2'},'k');
            legend('Forward Pts','Backward Pts','Forward','Backward','PolyxPoly',...
               'NumColumns',3,'Location','SouthOutside')
            legend('Forward Pts','Backward Pts','Forward','Backward',...
                 'NumColumns',3,'Location','SouthOutside')
end

end
