function [Plane, P] = Func_ComputeMinimumPlane(FunSys, param, IC, isVisual,  isDebug, opts)
%
%   [Plane, P] = Func_ComputeMinimumPlane(FunSys, param, IC, isVisual,  isDebug, opts)
%
%   Function that compute the minimum plane that can be reached during the
%   forward simulation. 
%
%   @param FunSys       : Function where the model to be simulated is
%                         implemented.
%   @param param        : Parameter structure with which simulate the model
%   @param IC           : Model initial condition to be tested.
%   @param isVisual     : Boolean parameter to activate the visualization.
%   @param isDebug      : Boolean parameter to enable the debug mode.
%   @param opts         : Visualization setting structure.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% =============================== HYPERPARAMETERS =========================
% Definition of certain variables and structures used inside some of the
% further section of the function.
options = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',@Func_EventODEAPSingle); 
nIC = size(IC,1); T = [0 450];


for i = 1:1:nIC 

    if isDebug
        fprintf('DEBUG - Forward Simulation - Elaborating %i/%i',i,nIC)
    end

    S = Func_Simulate(FunSys,T,IC(i,:)',param.VAR,param,options,1);
    [tMIN, j] = FindMinimum(S);

    if i == 1
        [MIN,j] = FindMinimum(S); 
        PlaneMinimum = Func_ComputePlane([0;0;1],MIN);
    end

    % ========================= VISUALIZATION =============================
    % Visualization of the simulation and the updated plane after having
    % executed the simulation.
    if isVisual
        fig = figure; 

        subplot(1,2,1)
        hold on
        plot(S.t,S.V,'LineWidth',1.2,'Color','k','LineStyle','-')
        plot(S.t(j),S.V(j),'LineWidth',1.2,'Color','g','LineStyle','none','Marker','.','MarkerSize',10)
        yline(MIN(2),'LineWidth',1.2,'Color','r','LineStyle','--')
        hold off

        Axes = gca; box on
        xlim([0 max(S.t)])
        ylim(opts.ylim)
        xlabel('$t$ [$ms$]','interpreter','latex')
        ylabel('$V$ [$mV$]','interpreter','latex')
        Func_SetAxesFont(Axes,opts)
        Func_SetGrid(Axes,opts)
        drawnow
    
        subplot(1,2,2)
        hold on,
        plot3(S.h,S.V,S.s,'LineWidth',1.2,'Color','k','LineStyle','-')
        plot3(S.h(j),S.V(j),S.s(j),'LineWidth',1.2,'Color','g','LineStyle','none','Marker','.','MarkerSize',10)
        Func_VisualizePlane2(PlaneMinimum,opts)
        hold off

        Axes = gca; box on
        Func_SetAxesFont(Axes,opts)
        Func_SetGrid(Axes,opts)
        Func_SetBoundaries(opts)
        zlim([0.4 0.5])
        view([-45 5])
        drawnow

        sgtitle(sprintf('IC - (%1.2f,%2.2f,%1.2f) [-,mV,-]',...
            IC(i,1),IC(i,2),IC(i,3)),'interpreter','latex')

        ANS = input('DEBUG - Continue?');
        if ANS == 0, return, end
    end

    if tMIN(3) > MIN(3)
        if isDebug
            fprintf(' - UPDATE - Prev.: %f - New: %f',MIN(3),tMIN(3))
        end
        MIN = tMIN;
        PlaneMinimum = Func_ComputePlane([0;0;1],MIN);
    end

    if isDebug
        fprintf('\n')
    end
end

Plane = PlaneMinimum;
P     = MIN;

    function [MIN, iAMINs] = FindMinimum(S)
        [~,iAMINs] = min(S.s); MIN = [S.h(iAMINs); S.V(iAMINs); S.s(iAMINs)];
    end

end