function L = Func_DetectSimulationBoundary(L,FuncSys,param,Func_Event,Direction,isVisual)
%
%   L = Func_DetectSimulationBoundary(L,FuncSys,param,Func_Event,Direction,isVisual)
%
%   Detection of the points in L for which it is possible to execute the
%   integration without exceeding a maximum norm limit. Cropping of the
%   initial condition line passed as input argument to guarantee the
%   bounded trajectory.
%
%   @param L            :   Initial condition line structure
%   @param FuncSys      :   Function handle to be simulated
%   @param param        :   Parameter structure
%   @param Func_Event   :   Event function to handle
%   @param Direction    :   Direction of the vector to scan
%   @param isVisual     :   Boolean variable to enable debugging
%                           visualization
%
%   @output L       :   Newly validated line of initial conditions.
%
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

options = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',Func_Event);
IC = zeros(length(L.V),3); IC(:,1) = L.V; IC(:,2) = L.h; IC(:,3) = L.s; nIC = size(IC,1);
index = true(nIC,1);

for i = 1:1:nIC
    if strcmp(Direction,'F')
        if isVisual
            fprintf('DEBUG - IC (%1.2f,%2.2f,%1.2f) [-,mV,-]\n',IC(i,2),IC(i,1),IC(i,3))
        end
        Si = Func_Simulate(FuncSys,[param.TL param.TH],IC(i,:)',param.VAR,param,options,1);
        if isVisual
            hold on; plot3(Si.h,Si.V,Si.s,'Color','m','LineWidth',1.5,'LineStyle','-'), hold off; drawnow,
        end
        if any(Si.ie == 1), index(i:end) = false; break; end
    end
    if strcmp(Direction,'B')
        if isVisual
            fprintf('DEBUG - IC (%1.2f,%2.2f,%1.2f) [-,mV,-]\n',IC(nIC-i+1,2),IC(nIC-i+1,1),IC(nIC-i+1,3))
        end
        Si = Func_Simulate(FuncSys,[param.TL param.TH],IC(nIC-i+1,:)',param.VAR,param,options,1);
        if isVisual
            hold on; plot3(Si.h,Si.V,Si.s,'Color','m','LineWidth',1.5,'LineStyle','-'), hold off; drawnow,
        end
        if any(Si.ie == 1), index(1:nIC-i+1) = false; break; end
    end
end

L = Func_Crop(L,index);

end