function L = Func_EnhanceLine(L,Geom,nP,Scale,a)
%
%   L = Func_EnhanceLine(L,Geom,nP,Scale,a)
%
%   Function that, once given a line L in input increases the number of
%   points that belongs to it. This is made with an integration step that
%   could follow different rationale specified inside the Scale parameter.
%
%   @param L    :   Line structure that should be enhanced.
%   @param Geom :   Gometry containing the dynamical system critical
%                   manifold.
%   @param nP   :   Number of points that should be produced inside the
%                   enhanced line.
%   @param Scale:   Update parameter scale. Choose between:
%                   ['Linear','Log','Poly','Exp']
%   @param a     :  Scaling hyperparameter.
%
%   @output L   :   Structure associated to the enhanced line.
%
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

isVertical = 0;   isHorizontal = 0;
LBV = min(L.V);   UBV = max(L.V);
C   = unique(L.C);

if length(unique(L.h)) == 1, isVertical   = 1; h = unique(L.h); end
if length(unique(L.s)) == 1, isHorizontal = 1; s = unique(L.s); end

L.h = []; L.V = []; L.s = [];

if isVertical
    L.V = linspace(LBV,UBV,nP);
    L.V = reMap(L.V,Scale);
    L.s = Geom.C0s(L.V,h);
    L.h = h*ones(nP,1);
    if C, L.C = true(nP,1); end
    if not(C), L.C = false(nP,1); end
end

if isHorizontal
    L.V = linspace(LBV,UBV,nP);
    L.V = reMap(L.V,Scale);
    L.h = Geom.C0h(L.V,s);
    L.s = s*ones(nP,1);
    if C, L.C = true(nP,1); end
    if not(C), L.C = false(nP,1); end
end

function reMapT = reMap(T,Transformation)
    switch Transformation
        case 'Log'
            x = linspace(1,a,nP);   % Reference axis
            R = log(x)/log(a);      % Reference axis - log[a]{x}
        case 'Exp'
            x = linspace(0,1,nP);   % Reference axis
            R = a.^x;               % Reference axis - a^{x}
        case 'Poly'
            x = linspace(0,1,nP);   % Reference axis
            R = x.^a;               % Reference axis - x^{a}
        case 'Root'
            x = linspace(0,1,nP);   % Reference axis
            R = x.^(1/a);           % Reference axis - x^{1/a}
        case 'Linear'
            R = linspace(0,1,nP);   % Reference axis - x
    end
    
    nT = length(T);             reMapT = zeros(nT,1);
    qT = T(1); AT = T(end)-qT;  qR = R(1); AR = R(end)-qR; 
    for i = 1:1:nT
        r = (R(i)-qR)/AR;
        reMapT(i) = AT*r+qT;
    end
end

end