function [ICCF, ICCB, IPS] = Func_FindIntersection(IF,IB, ICF, ICB, nC, isVisualProgress)
%
%   [ICCF, ICCB, IPS] = Func_FindIntersection(IF,IB, ICF, ICB, nC, isVisualProgress)
%
%   Function that compute the intersection between the forward and backward
%   simulation projection over the Sigma plane. This function estimate even
%   the initial conditions of the intersection points detected over the
%   plane and identify a number of IC equal to the number of canard that
%   are specified inside the nC parameter.
%
%   @param IF   :   Forward initial projection.
%   @param IB   :   Backward initial projection.
%   @param ICF  :   Initial condition of forward integration.
%   @param ICB  :   Initial condition of backward integration.
%   @param nC   :   Number of canard that needed to be identified.
%   @param isVisualProgress: Boolean to control the visualization of the
%                            algortihm progression.
%
%   @output ICCF:   Guess of initial condition associated to the canard for
%                   the forward integration.
%   @output ICCB:   Guess of initial condition associated to the canard for
%                   the backward integration.
%   @output IPS :   Structure containing the intersection points.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% ================================== INFORMATION ==========================
% Retrival of the information needed to run the code in a cleaner way.
n = length(IF); m = length(IB);

IFM = zeros(n,2); IBM = zeros(m,2);
for i = 1:1:n, IFM(i,:) = [IF{i}.x1, IF{i}.x2]; end
for i = 1:1:m, IBM(i,:) = [IB{i}.x1, IB{i}.x2]; end

% =============================== INTERSECTIONS ===========================
% Computation of the intersections through polyxpoly function after having 
% normalized and streatched the forward and backward projection curves over
% the plane.
MIN = min([IFM; IBM]);         MAX = max([IFM; IBM]);
IFM = Normalize(IFM,MIN,MAX);  IBM = Normalize(IBM,MIN,MAX);
[X,Y] = polyxpoly(IFM(:,1), IFM(:,2), IBM(:,1), IBM(:,2));  
nI = length(X); 

% ======================== IC ESTIMATION: FORWARD =========================
% Estimation of the initial condition via a simple linear fitting method
% for the forward simulation canard problem.
nK = 1;
for i = 1:1:nI    
    I    = [X(i), Y(i)];
    idx  = knnsearch(IFM,I,'K',2);
    if max(idx) == min(idx) + 1 && nK <= nC
        if isVisualProgress
            fprintf('DEBUG - FORWARD - Canard %i - Couple (%i,%i)\n', nK, max(idx), min(idx))
        end
        B.i  = idx;
        B.IC = ICF(idx,:);
        B.EP = IFM(idx,:);

        ICCF(nK,:) = EstimateIC(B,I);
        nK = nK + 1;
    end
end

% ======================== IC ESTIMATION: BACKWARD ========================
% Estimation of the initial condition via a simple linear fitting method
% for the forward simulation canard problem.
nK = 1;
for i = 1:1:nI
    I    = [X(i), Y(i)];
    idx  = knnsearch(IBM,I,'K',2);
    if max(idx) == min(idx) + 1 && nK <= nC
        if isVisualProgress
            fprintf('DEBUG - BACKWARD - Canard %i - Couple (%i,%i)\n',nK,max(idx),min(idx))
        end
        B.i  = idx;
        B.IC = ICB(idx,:);
        B.EP = IBM(idx,:);

        ICCB(nK,:)  = EstimateIC(B,I);
        nK = nK + 1;
    end
end

% ============================= OUTPUT ====================================
% Output preparation via the application of a DeNormalization procedure and
% a successive conversion of the same into a structure.
IP = [X,Y]; IP = DeNormalize(IP,MIN,MAX); nI = size(IP,1);
for i = 1:1:nI
    IPS{i}.x1 = IP(i,1);
    IPS{i}.x2 = IP(i,2);
end

% ============================== FUNCTION =================================
% Function to code readibility and handling of repetitive tasks.

    % Function - Normalize
    %            Normalize the vector x by streatching it respect to the 
    %            minimum and the maximum value passed as input to this
    %            function.
    function xN = Normalize(x,m,M)
        nP = size(x,1);
        xN = (x - repmat(m,[nP,1]))./repmat(M - m,[nP,1]);
    end

    % Function - DeNormalize
    %            Function that execute the inverse of the initially
    %            proposed normalization procedure.
    function xN = DeNormalize(x,m,M)
        nP = size(x,1);
        xN = x.*repmat(M - m,[nP,1]) + repmat(m,[nP, 1]);
    end

    % Function - Enhance Grid
    %            It creates, once given a vector x, the enhanced 
    %            version of the vector inserting between the different
    %            entries a number of eF points.
    function xN = EnhanceGrid(x,eF)
        nP = length(x);
        xN = zeros((nP-1)*eF+nP,1);
        for k = 1:1:length(x)-1
            xN(k+eF*(k-1):k+eF*k+1)=linspace(x(k),x(k+1),eF+2);
        end
    end

    % Function - Enhance Curve
    %            Enhance the curve passed as object to this function where
    %            the interpolation method exploited is an algorithm
    %            hyperparameter.
    function TN = EnhanceCurve(T,Method)
        eF = 10;
        nP = size(T,1); TT = T;       TT(:,3) = linspace(1,nP,nP);
        x  = TT(:,1);   y  = TT(:,2); z       = TT(:,3);
        
        zN = EnhanceGrid(z,eF);
        xN = interp1(z,x,zN,Method);
        yN = interp1(z,y,zN,Method);

        TN = [xN, yN];
    end

    % Function - Estimate Initial Conditions
    %            Estimate initial conditions associated to the
    %            intersection points between the two curves visualized.
    function IC = EstimateIC(B,I)
        [B.i, index] = sort(B.i); 
        B.IC = B.IC(index,:);
        B.EP = B.EP(index,:);

        PEP = norm(I - B.EP(1,:))/norm(B.EP(1,:) - B.EP(2,:));    % Percentage
        SF  = PEP*norm(B.IC(1,:) - B.IC(2,:));                    % Scaling factor
        MV  = (B.IC(2,:) - B.IC(1,:))/norm(B.IC(2,:) - B.IC(1,:));% Moving vector
        IC  = B.IC(1,:) + MV*SF;                                  % New IC
    end

end