function [ICCF, ICCB, IPS] = Func_FindIntersection2(Geom,param,IF,IB,ICF,ICB,nC,isVisualProgress)
%
%   [ICCF, ICCB, IPS] = Func_FindIntersection2(Geom,param,IF,IB,ICF,ICB,nC,isVisualProgress)
%
%   Function that compute the intersection between the forward and backward
%   simulation projection over the Sigma plane. This function estimate even
%   the initial conditions of the intersection points detected over the
%   plane and identify a number of IC equal to the number of canard that
%   are specified inside the nC parameter. Then these priors are corrected
%   via an optimization problem that minimizes the distance between the
%   projection of the simulations on the plane by optimizing the model's
%   forward and backward integration initial condition.
%
%   @param Geom :   Geomtry functions object.
%   @param param:   Parameter's structure.
%   @param IF   :   Forward initial projection.
%   @param IB   :   Backward initial projection.
%   @param ICF  :   Initial condition of forward integration.
%   @param ICB  :   Initial condition of backward integration.
%   @param nC   :   Number of canard that needed to be identified.
%   @param isVisualProgress: Boolean to control the visualization of the
%                            algorithm progression. Usually turned to true
%                            when debugging code.
%
%   @output ICCF:   Guess of initial condition associated to the canard for
%                   the forward integration.
%   @output ICCB:   Guess of initial condition associated to the canard for
%                   the backward integration.
%   @output IPS :   Structure containing the intersection points.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% ================================== INFORMATION ==========================
% Retrival of the information needed to run the code in a cleaner way.
n = length(IF); m = length(IB);

IFM = zeros(n,2); IBM = zeros(m,2);
for i = 1:1:n, IFM(i,:) = [IF{i}.x1, IF{i}.x2]; end
for i = 1:1:m, IBM(i,:) = [IB{i}.x1, IB{i}.x2]; end

% =============================== INTERSECTIONS ===========================
% Computation of the intersections through polyxpoly function after having 
% normalized and streatched the forward and backward projection curves over
% the plane.
MIN = min([IFM; IBM]);         MAX = max([IFM; IBM]);
IFM = Normalize(IFM,MIN,MAX);  IBM = Normalize(IBM,MIN,MAX);
[X,Y] = polyxpoly(IFM(:,1),IFM(:,2),IBM(:,1),IBM(:,2));  
nI = length(X); 

% ======================== IC ESTIMATION: FORWARD =========================
% Estimation of the initial condition via a simple linear fitting method
% for the forward simulation canard problem.
nK = 1;
for i = 1:1:nI    
    I    = [X(i), Y(i)];
    idxF = knnsearch(IFM,I,'K',2); idxB = knnsearch(IBM,I,'K',2);

    if max(idxF) == min(idxF) + 1 && max(idxB) == min(idxB) + 1 && nK <= nC

        if isVisualProgress % Debug Purposes
            fprintf('DEBUG - Both - Canard %i - Couple Forward (%i,%i) - Couple Backward (%i,%i)\n',...
                nK,max(idxF),min(idxF),max(idxB),min(idxB))
        end
        
        % Forward            % Backward
        B.iF = idxF;         B.iB = idxB;
        B.ICF = ICF(idxF,:); B.ICB = ICB(idxB,:);
        B.EPF = IFM(idxF,:); B.EPB = IBM(idxB,:);

        % Forward and Backward IC Optimization
        [ICCF(nK,:), ICCB(nK,:)] = EstimateIC(Geom,param,B,I);
        
        nK = nK + 1;
    end
end

% ============================= OUTPUT ====================================
% Output preparation via the application of a DeNormalization procedure and
% a successive conversion of the same into a structure.
IP = [X,Y]; IP = DeNormalize(IP,MIN,MAX); nI = size(IP,1);
for i = 1:1:nI
    IPS{i}.x1 = IP(i,1);
    IPS{i}.x2 = IP(i,2);
end

% ============================== FUNCTION =================================
% Function to code readibility and handling of repetitive tasks.

    % Function - Normalize
    %            Normalize the vector x by streatching it respect to the 
    %            minimum and the maximum value passed as input to this
    %            function.
    function xN = Normalize(x,m,M)
        nP = size(x,1);
        xN = (x - repmat(m,[nP,1]))./repmat(M - m,[nP,1]);
    end

    % Function - DeNormalize
    %            Function that execute the inverse of the initially
    %            proposed normalization procedure.
    function xN = DeNormalize(x,m,M)
        nP = size(x,1);
        xN = x.*repmat(M - m,[nP,1]) + repmat(m,[nP, 1]);
    end

    % Function - Enhance Grid
    %            It creates, once given a vector x, the enhanced 
    %            version of the vector inserting between the different
    %            entries a number of eF points.
    function xN = EnhanceGrid(x,eF)
        nP = length(x);
        xN = zeros((nP-1)*eF+nP,1);
        for k = 1:1:length(x)-1
            xN(k+eF*(k-1):k+eF*k+1)=linspace(x(k),x(k+1),eF+2);
        end
    end

    % Function - Enhance Curve
    %            Enhance the curve passed as object to this function where
    %            the interpolation method exploited is an algorithm
    %            hyperparameter.
    function TN = EnhanceCurve(T,Method)
        eF = 10;
        nP = size(T,1); TT = T;       TT(:,3) = linspace(1,nP,nP);
        x  = TT(:,1);   y  = TT(:,2); z       = TT(:,3);
        
        zN = EnhanceGrid(z,eF);
        xN = interp1(z,x,zN,Method);
        yN = interp1(z,y,zN,Method);

        TN = [xN, yN];
    end

    % Function - Estimate Initial Conditions
    %            Estimate initial conditions associated to the
    %            intersection points between the two curves visualized.
    %            This estimate function optimize the initial condition
    %            chosen inside the two curves to reconstruct via
    %            respectively backward and forward solution of the
    %            dynamical system the unstable and stable manifold of the
    %            system around the fold point.
    function [ICF, ICB] = EstimateIC(Geom,param,B,I)

        % Initial Estimate - Forward solution
        %                    Two successive points inside the curve of
        %                    forward integration initial condition inside
        %                    which is contained the intersection.
        [B.iF, index] = sort(B.iF); 
        B.ICF = B.ICF(index,:);
        B.EPF = B.EPF(index,:);

        % Initial Estimate - Backward solution
        %                    Two successive points inside the curve of
        %                    backward integration initial condition inside
        %                    which is contained the intersection.
        [B.iB, index] = sort(B.iB); 
        B.ICB = B.ICB(index,:);
        B.EPB = B.EPB(index,:);

        % Options - Definition
        %           The event to detect in both the boundary value problems
        %           that will be solved to terminate the integration of
        %           both backward and forward projection is the one of
        %           belonging to the plane Sigma.
        optionsODE    = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',@Func_EventODEDouble);
        param.options = optionsODE;
        optionsOPTIM  = optimset('TolX',1e-12,'TolFun',1e-12,'Display','off',...
                            'Algorithm','interior-point','MaxIter',100);
        
        % Model's geometry - Definition
        %                    Introduction of the two curves definition
        %                    inside the parameter structure that will be
        %                    then passed to the optimization problem.
        param.C0h = Geom.C0h; param.s0 = B.ICB(1,3);
        param.C0s = Geom.C0s; param.h0 = B.ICF(1,2);

        % Optimization problem - Parameters
        %              Definition of the optimization problem
        %              hyperparameters to generate the figures and produce
        %              the results that neeed to be reconstructed. The
        %              initial estimate are for both the forward and
        %              backward integration the half point in between the
        %              two successively identified.
        p0  = [mean(B.ICF(:,1)); mean(B.ICB(:,1))];
        pLB = [min(B.ICF(:,1)); min(B.ICB(:,1))];
        pHB = [max(B.ICF(:,1)); max(B.ICB(:,1))];

        if isVisualProgress % Debug Purposes
            fprintf('DEBUG - Starting Optimization...\n')
        end
        % Optimization problem - Definition
        %               Definition of the optimization problem that should
        %               be solved which consists in minimizing the distance
        %               between the points belonging to the plane Sigma
        %               previously identified.
        pH = fmincon(@(p)Func_CostCanard(p,param),p0,[],[],[],[],pLB,pHB,[],optionsOPTIM);
        if isVisualProgress % Debug Purposes
            fprintf('DEBUG - Optimization Completed!\n')
        end

        % Output - Definition
        %          Definition of the output structure that will be passed
        %          on the main function workflow.
        VFH = pH(1); ICF = [VFH, param.h0, Geom.C0s(VFH,param.h0)];
        VBH = pH(2); ICB = [VBH, Geom.C0h(VBH,param.s0), param.s0];
    end

end