function [Geom,EQ,FS] = Func_LAV3DGeometry(param,state)
%
%   [Geom,EQ,FS] = Func_LAV3DGeometry(param,state)
%
%   Function that compute the LAV 3D Model geometry. The results are
%   directly passed as output of the function.
%
%   @param param        :   Parameter structure representative of the LAV
%                           3D model that should be analysed.
%   @param state        :   Parameter that enable the update visualization.
%
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

% ========================= MODEL PARAMETERS ==============================
% Definition of the model parameters to perform the calculation in the
% following part of the dedicated function.
paramFields = fieldnames(param);
for i = 1:1:length(paramFields)
    if strcmp(paramFields{i},'r'), eval(['rT = param.',paramFields{i},';']), end
    if strcmp(paramFields{i},'w'), eval(['wT = param.',paramFields{i},';']), end
    if strcmp(paramFields{i},'IApp'), eval(['IAppT = param.',paramFields{i},';']), end
    if not(strcmp(paramFields{i},'w')) && not(strcmp(paramFields{i},'r')) && not(strcmp(paramFields{i},'IApp'))
        eval([paramFields{i},'= param.',paramFields{i},';'])
    end
end

%  ======================= MATLAB FUNCTIONs ===============================
% Functions that evaluate the m, n, h, s transition rates.
fAm = @(V) (1.86*(V + 25.4))./(1 - exp(-(V + 25.4)/10.3));
fBm = @(V) (-0.086*(V + 29.7))./(1 - exp((V + 29.7)/9.16));
fAh = @(V) (-0.0336*(V + 118))./(1 - exp((V + 118)/11));
fBh = @(V) 2.3./(1 + exp(-(V + 35.8)/13.4));
fAn = @(V) (0.186*(V + 48.4))./(1 - exp(-(V + 48.4)/10.3));
fBn = @(V) (-0.0086*(V + 42.7))./(1 - exp((V + 42.7)/9.16));
fAs = @(V) (0.00122*(V + 19.5))./(1 - exp(-(V + 19.5)/23.6));
fBs = @(V) (-0.000739*(V + 87.1))./(1 - exp((V + 87.1)/21.8));

% ========================= MATLAB SYMBOLIC ===============================
syms V s h IApp r w
% ======================== ACTIVATION VARIABLES ===========================
% System activation variables expressed through symbolic notation to
% perform the further calculations and inspect the system geometry.
Am   = symfun(sym(fAm),V);    Bm  = symfun(sym(fBm),V);   
An   = symfun(sym(fAn),V);    Bn  = symfun(sym(fBn),V);   
Ah   = symfun(sym(fAh),V);    Bh  = symfun(sym(fBh),V);   
As   = symfun(sym(fAs),V);    Bs  = symfun(sym(fBs),V);             
mInf = symfun(Am/(Am+Bm),V); nInf = symfun(An/(An+Bn),V);
hInf = symfun(Ah/(Ah+Bh),V); hTau = symfun(1/(Ph*(Ah+Bh)),V);
sInf = symfun(As/(As+Bs),V); sTau = symfun(1/(Ps*(As+Bs)),V);
% ======================== REFERENCE MODEL ================================
% Symbolic notation for the dynamical system definition.
if state, fprintf('DEBUG - Model Computation ... '), end
INaF = symfun(gNaF*(mInf^3)*h*(V-ENa),[V h]);
INaP = symfun(gNaP*(nInf^3)*(V-ENa),V);
IKS  = symfun(gKS*s*(V-EK),[V,s]);
IL   = symfun(gL*(V-ELK),V);
IHCN = symfun(gHCN*r*(V-EHCN),V);
IM   = symfun(gM*w*(V-EK),V);
f = symfun((-INaF(V,h)-INaP(V)-IKS(V,s)-IHCN(V)-IM(V)-IL(V)+IApp)/Cm,[V h s IApp]);
g1 = symfun((hInf - h)/hTau,[V h]);
g2 = symfun((sInf - s)/sTau,[V s]);
if state, fprintf('Completed!\n'), end
% =========================== CRITICAL MANIFOLD ===========================
% Critical manifold symbolic computation.
if state, fprintf('DEBUG - Critical Manifold Computation ... '), end
C0s = solve(f == 0, s);
C0h = solve(f == 0, h);
if state, fprintf('Completed!\n'), end
% =============================== DERIVATIES ==============================
% Derivatives computation of the function f (fast variable V) respect to
% the three invovled system dynamical variables h,s and V.
if state, fprintf('DEBUG - f Differentiation Computation ... '), end
dhf = diff(f,h); dsf = diff(f,s); dVf = diff(f,V);
if state, fprintf('Completed!\n'), end
% ========================== DESINGULARIZED MODEL =========================
% Symbolic desingularization of the system.
if state, fprintf('DEBUG - Desingularized Model ... '), end
F = symfun(dhf(V,h,s,IApp)*g1(V,h)+dsf(V,h,s,IApp)*g2(V,s),[V h s IApp]);
F = subs(F,s,C0s);
H = symfun(-dVf(V,h,s,IApp)*g1(V,h),[V h s IApp]);
H = subs(H,s,C0s);
if state, fprintf('Completed!\n'), end
% ============================= FOLD CURVE ================================
% Symbolic computation of the fold curve.
if state, fprintf('DEBUG - Fold Curve Computation ... '), end
L = solve(subs(dVf,s,C0s) == 0,h);
if state, fprintf('Completed!\n'), end
% ====================== NORMAL SWITCHING CONDITION =======================
% Symbolic computation of the Normal Switching Condition.
if state, fprintf('DEBUG - Normal Switching Condition Computation ... '), end
NSW = solve(F == 0, h);
if state, fprintf('Completed!\n'), end
% ============================ EQUILIBRIUM POINT ==========================
% Numerical computation of the system equilibrium point.
if state, fprintf('DEBUG - Equilibrium Point Computation ... '), end
EQ.V = double(vpasolve(subs(f,{h s r w IApp},{hInf sInf rT wT IAppT}) == 0));
EQ.s = double(vpa(subs(sInf,{V},{EQ.V})));
EQ.h = double(vpa(subs(hInf,{V},{EQ.V})));
if state, fprintf('Completed!\n'), end
% ============================ FOLDED SINGULARITY =========================
% Numerical computation of the system folded singularity close to the
% dynamical system equilibrium point.
if state, fprintf('DEBUG - Folded Singularity Computation ... '), end
FS.V = double(vpasolve(subs(NSW,[r w IApp],[rT wT IAppT]) == subs(L,[r w IApp],[rT wT IAppT]),-80));
FS.h = double(vpa(subs(NSW,[V r w IApp],[FS.V rT wT IAppT])));
FS.s = double(vpa(subs(C0s,[V h r w IApp],[FS.V FS.h rT wT IAppT])));
if state, fprintf('Completed!\n'), end
% ============================== CLASSIFICATION ===========================
% Classification of both equilibrium and folded singularity based on the
% eigenvalues derived.
if state, fprintf('DEBUG - Jacobian and Eigenvalues Computation ...'), end
EQ.JEQ = jacobian([f(V,h,s,IApp),g1(V,h),g2(V,s)],[V,h,s]);
FS.JFS = jacobian([F(V,h,s,IApp),H(V,h,s,IApp)],[V,h]);
EQ.JEQ = subs(EQ.JEQ,[w r IApp],[wT rT IAppT]);
FS.JFS = subs(FS.JFS,[w r IApp],[wT rT IAppT]);
EQ.E = eig(vpa(subs(EQ.JEQ,[V,h,s],[EQ.V,EQ.h,EQ.s])));
FS.E = eig(vpa(subs(FS.JFS,[V,h,s],[FS.V,FS.h,FS.s])));
if state, fprintf('Completed!\n'), end

% ========================= MATLAB FUNCTIONs CONVERSION ===================
% Generation of the Geometry structure which contains the functions
% associated to the critical manifold C0, the fold curve L and the Normal
% Switching Condition.
Geom.C0s  = matlabFunction(subs(C0s,[r w IApp],[rT wT IAppT]));
Geom.C0h  = matlabFunction(subs(C0h,[r w IApp],[rT wT IAppT]));
Geom.L   = matlabFunction(subs(L,[r w IApp],[rT wT IAppT]));
Geom.NSW = matlabFunction(subs(NSW,[r,w,IApp],[rT,wT,IAppT]));
Geom.dVf = matlabFunction(subs(dVf,[r w IApp],[rT wT IAppT]));

end