function [S,e] = Func_OptimizeICSolution(Geom, param, CCLB, CCLF, isVisualProgress, Event)
%
%   [S,e] = Func_OptimizeICSolution(Geom, param, CCLB, CCLF, isVisualProgress, Event)
%
%   Function that search for the best optimal condition to simulate the
%   model such that the most suitable canard orbit approximation is
%   retrived. Together with that even the approximation error is proposed.
%
%   @param Geom     :   Model's geometry structure.
%   @param param    :   Model's parameters structure.
%   @param CCLB     :   Backward integration initial conditions.
%   @param CCLF     :   Forward integration initial conditions.
%   @param isVisualP:   Boolean which controls the code progresses.
%   @param Event    :   Function that trigger the integration termination
%
%   @output S       :   Simulation approximation.
%   @output e       :   Simulation approximation error.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% ========================== OPTIONS: DEFINITION ==========================
% The event to detect in both the boundary value problems that will be 
% solved to terminate the integration of both backward and forward 
% projection is the one of belonging to the plane Sigma.
optionsODE = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',Event);
param.options = optionsODE;
optionsOPTIM  = optimset('TolX',1e-20,'TolFun',1e-20,'Display','off',...
    'Algorithm','interior-point','MaxIter',500);

% ========================== GEOMETRY: DEFINITION =========================
% Introduction of the two curves definition inside the parameter structure
% that will be then passed to the optimization problem.
param.C0h = Geom.C0h; param.s0 = CCLB.s(end);
param.C0s = Geom.C0s; param.h0 = CCLF.h(end);

% ========== OPTIMIZATION PROBLEM: SOLUTION SPACE AND BOUNDARIES ==========
% Definition of the optimization problem hyperparameters to generate the
% figures and produce the results that neeed to be reconstructed. The 
% initial estimate are for both the forward and backward integration the
% half point in between the two successively identified.
p0  = [mean(CCLB.V); mean(CCLF.V)];
pUB = [max(CCLB.V); max(CCLF.V)];
pLB = [min(CCLB.V); min(CCLF.V)];

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
if isVisualProgress
    fprintf('DEBUG - Starting Optimization...\n')
end

% ===================== OPTIMIZATION PROBLEM: DEFINITION ==================
% Definition of the optimization problem that should be solved which
% consists in minimizing the distance between the points belonging to the
% plane Sigma previously identified.
pH = fmincon(@(p)Func_CostCanardApproximation(p,param),p0,[],[],[],[],pLB,pUB,[],optionsOPTIM);

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
if isVisualProgress
    fprintf('DEBUG - Optimization Completed!\n')
end

% ========================== OUTPUT: SHAPING ==============================
% Definition of the output structure that will be passed on the main 
% function workflow.
VFH = pH(2); ICF = [VFH, param.h0, Geom.C0s(VFH,param.h0)];
VBH = pH(1); ICB = [VBH, Geom.C0h(VBH,param.s0), param.s0];

% ============ APPROXIMATED CANARD: FORWARD & BACKWARD SIMULATION =========
% Simulation of the dynamical system with forward and backward integration
% starting from the optimized initial conditions searched via the previous
% problem.
param.D = param.DF; T = [0 100];  SF = Func_Simulate(@Func_LAV_3D,T,ICF,param.VAR,param,optionsODE,true);
param.D = param.DB; T = [0 -100]; SB = Func_Simulate(@Func_LAV_3D,T,ICB,param.VAR,param,optionsODE,true);

% ===================== APPROXIMATED CANARD: RESHAPING ====================
% Merge of the solution obtained via the forward and backward integrated
% problem. The last point reached in the backward computation is neglected.
nTF = length(SF.V); nTB = length(SB.V)-1; S.t = zeros(nTF+nTB,1);     S.i = [nTF, nTF+1];
S.V = [SF.V; flipud(SB.V(1:nTB))]; S.h = [SF.h; flipud(SB.h(1:nTB))]; S.s = [SF.s; flipud(SB.s(1:nTB))];

% ===================== APPROXIMATED CANARD: TIME =========================
% Construction of the vector associated to the approximated canard solution
S.t(1:nTF) = SF.t; t = flipud(SB.t); t = t + abs(t(1));
for iT = nTF+1:1:nTF+nTB, S.t(iT) = S.t(iT-1) + (t(iT-nTF+1) - t(iT-nTF)); end

% ========================= APPROXIMATION ERROR ===========================
% Retrival of both forward and backward end points. Further computation of
% the approximation error made by trminating the integration in
% correspondence of the specified plane.
EPF = [SF.V(end); SF.h(end); SF.s(end)]; EPB = [SB.V(end); SB.h(end); SF.s(end)]; e = norm(EPF - EPB);

end