function Axes = Func_VisualizePlaneProjection3(S,STYLE,EVENT,IdxToVisualize)
%
%   Axes = Func_VisualizePlaneProjection3(S,STYLE,EVENT,IdxToVisualize)
%
%   Function that visualize the projection of the passed simulation
%   structure onto a given plane specified as additional external event.
%   The style of the visualization can be changed depending on the kind of
%   information that can be displayed.
%
%   @param S             :  Structure that contains the data that needed to be
%                           visualized.
%   @param STYLE         :  Style input parameter of the data that should be
%                           visualized. 1 indicates the attracting branch of the
%                           reconstructed slow manifold. 2 indicates the repelling 
%                           branch of the slow manifold, instead 3 and 4
%                           respectively the reconstructed forward and backward 
%                           canard solutions.
%   @param EVENT         :  Event that needed to be visualized.
%   @param IdxToVisualize:  Cell structure that needed to be visualized.  
%
%   @output Axes:   Axes object.
%
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

% ======================== HYPERPARAMETERS: RETRIVAL ======================
% Retrival of the hypeparameters further used in the code.
nS = length(S); x  = zeros(nS,1); y  = zeros(nS,1);
if nargin == 3, IdxToVisualize = ones(length(S),1); end

% ============================ VISUALIZATION ==============================
% Visualization of the information resepct to the chosen style passed as
% input to this function.
switch STYLE
    case 1  % Attracting branch of the slow manifold
        [x,y] = Func_RetriveFromSlowManifold(S,EVENT,IdxToVisualize);
        opts = Func_SetVisualizationSettings('r','-',1,'none',5);
        Func_Visualize(x,y,opts)
    case 2  % Repelling branch of the slow manifold
        [x,y] = Func_RetriveFromSlowManifold(S,EVENT,IdxToVisualize);
        opts = Func_SetVisualizationSettings('k','-',1,'none',5);
        Func_Visualize(x,y,opts)
    case 3 % Forward integrated canard orbits
        [x,y] = Func_RetriveFromCanard(S);
        nS = length(x); COL = Func_ColorMapCanard(nS); 
        opts = Func_SetVisualizationSettings(COL,'none',1,'o',3);
        Func_Visualize2(x,y,opts)
    case 4 % Backward integrated canard orbits
        [x,y] = Func_RetriveFromCanard(S);
        nS = length(x); COL = Func_ColorMapCanard(nS);
        opts = Func_SetVisualizationSettings(COL,'none',1,'^',3);
        Func_Visualize2(x,y,opts)
end

% ======================== INFORMATION: RETRIVAL ==========================
% Retrival of additional information necessary to be handled outside this
% function.
Axes = gca;

% ======================== ADDITIONAL FUNCTIONS ===========================
% Additional functions defined to improve the code readibility.

    function [x,y] = Func_RetriveFromSlowManifold(S,EVENT,IdxToVisualize)
        nS = length(S); idx = []; x = []; y = []; c = 1;
        for iS = 1:1:nS
            if IdxToVisualize(iS) == 1
                idx = find(S{iS}.ie == EVENT); idx = idx(end);
                x(c) = eval(['S{iS}.se(idx,1)']);
                y(c) = eval(['S{iS}.se(idx,2)']);
                c = c + 1;
            end
        end
    end

    function [x,y] = Func_RetriveFromCanard(S)
        nS = length(S); idx = []; x = zeros(nS,1); y = zeros(nS,1);
        for iS = 1:1:nS
            x(iS) = eval(['S{iS}.V(end)']);
            y(iS) = eval(['S{iS}.h(end)']);
        end
    end

    function Func_Visualize(x,y,opts)
        hold on
        plot(x,y,'Color',opts.COL,'LineStyle',opts.LINESTYLE,'LineWidth',opts.LINEWIDTH,...
            'Marker',opts.MARKER,'MarkerSize',opts.MARKERSIZE)
        hold off
    end

    function Func_Visualize2(x,y,opts)
        nS = length(x);
        for iP = 1:1:nS
            hold on
            plot(x(iP),y(iP),'Color',opts.COL(iP,:),'LineStyle',opts.LINESTYLE,'LineWidth',opts.LINEWIDTH,...
                'Marker',opts.MARKER,'MarkerSize',opts.MARKERSIZE,'MarkerFaceColor',opts.COL(iP,:))
            hold off
        end
    end

    function opts = Func_SetVisualizationSettings(COL,LINESTYLE,LINEWIDTH,MARKER,MARKERSIZE)
        opts.COL = COL;
        opts.LINEWIDTH = LINEWIDTH;
        opts.LINESTYLE = LINESTYLE;
        opts.MARKER = MARKER;
        opts.MARKERSIZE = MARKERSIZE;
    end

end