function [Axes, p, L] = Func_VisualizeSlowManifold2(S,Type,VAR,EL,COL)
%
%   [Axes, p, L] = Func_VisualizeSlowManifold2(S,Type,VAR,EL)
%
%   Function that visualize the slow manifold content inside the S
%   structure passed as input to the function. The visualization occurs
%   inside a 3D phase space where the x,y,z axes are set up with the
%   variables specified inside VAR.
%
%   @param S    :   Slow manifold structure
%   @param Type :   Type of slow manifold that should be visualized. This
%                   parameter could be set to 'A' or 'R' respectively for
%                   attracting and repelling subsets.
%   @param VAR  :   Variables sets that specify the x, y and z axes of the
%                   opened graphic window.
%   @param EL   :   Boolean parameter that control the End Line
%                   visualization.
%
%   @output Axes:   Axes object output of the function
%   @output p   :   Line handle of the visualized slow manifold
%   @output L   :   Legend string associated to the kind of slow manifold
%                   visualized.
%
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

% =========================== INFORMATION =================================
% Extraction of information used inside the following lines of code.
nT = length(S{1}.t); nS = length(S);

% =========================== COLOR SELECTION =============================
% Selection of the color with which the manifold will be visualized through
% some dedicated indicators and markers. Definition of the slow manifold
% legend too.
if strcmp(Type,'A'), COL = 'r'; L = '$S_{a}^{\epsilon}$'; end
if strcmp(Type,'R'), COL = 'k'; L = '$S_{r}^{\epsilon}$'; end
if strcmp(Type,'RS'), nSAO = S{1}.nSAO; L = sprintf('$R_{%i}$',nSAO); end

% =========================== PREPROCESSING ===============================
% Preprocessing of the data contained inside the S structure via some
% interpolation to get the same amount of data and create a handlable
% structure.
for i = 1:1:nS
    temp_nT = length(S{i}.t);
    if not(nT > temp_nT), nT = temp_nT; end
end
for i = 1:1:nS, S{i} = downSampling(S{i},nT,VAR); end
SS = surfaceConversion(S,VAR);

% ========================== MANIFOLD VISUALIZATION =======================
% Visualization of the manifold previously computed with the surf command.
% Depending on the entries, even the last points can be displayed via a
% connection line dedicately opened.
hold on, VARVisualize = {'x','y','z'};
for iV = 1:1:length(VARVisualize), eval([VARVisualize{iV},'= SS.',VAR{iV},';']), end
hold on, p = surf(x,y,z,'FaceColor',COL,'FaceAlpha',0.125,'EdgeColor','none'); hold off

% ======================= PLANE PROJECTION VISUALIZATION ==================
% Highlighting of the last simualtions value projected inside a didicated
% plane.
if EL
    x = zeros(length(S),1); y = zeros(length(S),1); z = zeros(length(S),1);
    for i = 1:1:nS, x(i) = S{i}.h(end); y(i) = S{i}.V(end); z(i) = S{i}.s(end); end
    hold on, plot3(x,y,z,'Color',COL,'LineWidth',1.5,'LineStyle','-'), hold off
end

% =========================== INFORMATION: RETRIVAL =======================
% Retrival of the information associated to the axes where currently the
% information have been displayed.
Axes = gca;

% Function - downSampling(S,nT)
%            Function that execute a downsampling or an upsampling of the 
%            structure with all its related fields passed as input. The
%            number of data that will be retained is equal to the passed
%            parameter nT.
function SN = downSampling(S,nT,VAR)
    SN.t  = linspace(S.t(1),S.t(end),nT);
    for j = 1:1:length(VAR)
        eval(['SN.',VAR{j},' = interp1(S.t,S.',VAR{j},',SN.t);'])
    end
end

% Function - surfaceConversion(S)
%            Conversion of the generic structure S into a matrix object
%            that will be used to be visualized.
function SS = surfaceConversion(S,VAR)
    nSIM = length(S);      
    n  = length(S{1}.t); 
    for j = 1:1:length(VAR)
        eval(['SS.',VAR{j},' = zeros(nSIM,n);'])
    end
    for j = 1:1:nSIM
        for k= 1:1:length(VAR)
            eval(['SS.',VAR{k},'(j,:) = S{j}.',VAR{k},';'])
        end
    end
end

end