%
% Canard Identification in Slow Manifolds Intersection
%
% ---- Note ---------------------------------------------------------------
% Algorithm used to recognize those canard orbits laying in the
% intersections of the approximated attracting and repelling slow 
% manifolds.
%
% ---- Requirements -------------------------------------------------------
% It requires the following folder with the data made available to work
% properly. Specifically, it needs:
%
%   -   Functions:  Folder containing the functions used in the code.
%   -   Figures  :  Folder where to export figures.
%   -   Data     :  Folder storing the required data.
%
% Non-Standard Toolbox: Mapping Toolbox
%
% ---- Authors ------------------------------------------------------------
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

%% PREPARATION
% Preparation of the file's workspace and MATLAB's enviroment.
close all; clear all; clc;

%% FUNCTIONS
% Add to path the function's folder where all the different functions are
% contained.
addpath('../Function/')

%% PATH
% Path of where are stored the different .mat file which are produced with
% this program and can be used to perform or not computations.

pathGeometry             = '../Data/Geometry/';
pathSlowManifold         = '../Data/SlowManifold/BVP/';
pathSlowManifoldSettings = '../Data/SlowManifold/Settings/';
path3DSimulation         = '../Data/3DSimulations/';
path5DSimulation         = '../Data/5DSimulations/';
pathOptions              = '../Data/Options/';
pathCanard               = '../Data/Canard/';

%% MODEL'S PARAMETER
% Definition of the model's parameter to avoid to define them inside the
% code and generating problem in the different simulations

% ================================ SETTINGS ===============================
% Settings used to select which kind of experiment to consider of which
% reconstruct the slow manifold.
iEXP     = input('EXP - Select Experiment: [1-5]');                     % Experiments
iCurrent = input('EXP - Select Current: [1 - Low | 2 - High]');         % Current
CTRL     = input('EXP - Select State: [Control | Stimulated]','s');     % Control/Stimulated

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('PREPARATION - Experiment Information Retrival...')

% ============================= EXPERIMENT ================================
% Experiment information extraction. These will be then used to tune where
% to reconstruct the Slow Manifold in order to explain the dynamical system
% model behaviour.
fileNameSimulation = sprintf('LAV_3D_Simulation_Exp%i.mat',iEXP);
load([path3DSimulation,fileNameSimulation]);
EXP        = eval(['EXP',num2str(iEXP),';']);
param      = eval(['EXP.',CTRL,'.param;']);
param.r    = eval(['EXP.',CTRL,'.rAVG(iCurrent)*EXP.',CTRL,'.KHCN;']);
param.w    = eval(['EXP.',CTRL,'.wAVG(iCurrent)*EXP.',CTRL,'.KM;']);
param.IApp = eval(['EXP.',CTRL,'.IApp(iCurrent);']);
S          = eval(['EXP.',CTRL,'.S{iCurrent};']);

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('PREPARATION - File Name Construction...')

% ========================= FILE NAME: DEFINITION =========================
% Definition of the main file names that can be used in further section of
% the code to improve the code readibility.
fileNameGeometry     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Geometry'];
fileNameSettings     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_BVPSettings'];
fileNameSlowManifold = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_SlowManifold'];
fileNameOptions      = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Options'];
fileNameCanard       = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Canard'];

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('PREPARATION - Visualization Setting Loading...')

% ========================= VISUALIZATION SETTINGS ========================
% Settings exploited to execute correctly the further proposed
% visualizations. These settings will be exploited in the further sections
% of the program.
load([pathOptions,fileNameOptions]);
opts.Font = 'Times New Roman'; 
opts.FontSize = 12;

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('PREPARATION - Experiment adaptation...')

% ======================== EXPERIMENT: INFORMATION ========================
% Extraction of information that will be used to reconstruct the slow
% manifold in the further section of the code.
T = 350;                                % Cropping time
S = Func_CropTime(S,T);                 % Cropping simulation structure
eval(['EXP.',CTRL,'.S{iCurrent} = S;']) % Cropping experiment structure
[~,I] = min(S.V); mSh = S.h(I);
I = find(S.V>0);  mSs = mean(S.s(I));

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')

%% MODEL'S VARIABLE
% Definition of the model's variable that will be inspected to perform all
% the possible analysis over the considered model.

VAR3D = {'h','V','s'}; ACT3D = {'h','s'}; param.VAR = {'V','h','s'};

%% MODEL'S BOUNDARIES
% Boundaries necessary to perform all the different simulations and
% visualizations. Act on this structure involves changes respectevely
% inside visualization and boundaries of computed curves or surfaces.

B.t = [0 50]; B.V = [-83.8 80]; B.h = [0 1]; B.s = [0 1];

%% SLOW MANIFOLD
% Loading of the slow manifold computed .mat file. The files are loaded and
% then are manipulated in further step of the analysis. Loading even of the
% model's geometry.

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('GEOMETRY - Loading...')

% ======================= LOADING: MODEL'S GEOMETRY =======================
% Loading of the model's geometry inside the matlab workspace to be used in
% further computation and eventual visualizations.
load([pathGeometry,fileNameGeometry])

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('SLOW MANIFOLD - Loading...')

% ======================= LOADING: COMPUTED SLOW MANIFOLD =================
% Loading of the slow manifold computed in another .m file produced via
% Matlab. This is foundamental to perform the analyses proposed further.
load([pathSlowManifold,fileNameSlowManifold])

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('IC - Reconstruction...')

% ============================ ICs: RECONSTRUCTION ========================
% Reconstruction of the ICs used to solve the BVP in another file. These
% are necessary to reconstruct the canard orbits of the slow manifold.
ICF(:,1) = LF.V; ICF(:,2) = LF.h; ICF(:,3) = LF.s;
ICB(:,1) = LB.V; ICB(:,2) = LB.h; ICB(:,3) = LB.s;

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')

%% CANARD - COMPUTATION
% Computation of the canard orbits respect to the plane which passes
% through the folded node slightly delocalized towards lower region by a
% specific factor.

REQUEST = true;

if input('CANARD - Would you recompute the canard orbits? [0 - No | 1 - Yes] ') == 1
    while REQUEST
        clear ICCF_OP ICCB_OP IP_OP Error_OP
        clear ICCF_LM ICCB_LM IP_LM Error_LM
        clear SCF_LM SCB_LM SCF_OP SCB_OP IF IB
        % ======================== CANARD: NUMBER OF CANARD ===================
        % Number of canard that needed to be identified during the execution of
        % the following code section. This is necessary to infer the model's 
        % behaviour and the evolution of the dynamic.
        nC = input('CANARD - How many canard do you want to identify?');
        
        % ====================== HYPEPARAMETERS: DEFINITION ===================
        % Definition of the foundamental hypeparameters that will be used to 
        % better handle the further proposed functions.
        param.SigmaFN = SigmaFNT; param.SigmaDW = SigmaDW; param.Sigma = SigmaFNT;
        param.options = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',@Func_EventODEDouble);
        
        % ========================= MARKER: CODE STATUS =======================
        % Marker use to track the code status. These print information inside 
        % cmd.
        fprintf('CANARD - Plane Intersection Computation...')
        
        % =================== COMPUTATION: PLANE INTERSECTION =================
        % Computation of the plane intersection. The plane considered to 
        % compute the intersections is the one that passes through the folded 
        % node and it is slightly move in lower position respect to this last.
        IF = Func_ComputePlaneIntersection(SigmaFNT,SF,PFST,1,0);
        IB = Func_ComputePlaneIntersection(SigmaFNT,SB,PFST,1,0);
        
        % ========================= MARKER: CODE STATUS =======================
        % Marker use to track the code status. These print information inside 
        % cmd.
        fprintf('Completed!\n')
        fprintf('CANARD - Canard Computation with Linear Method...')
        
        % ===================== CANARDS: LINEAR METHOD ========================
        % Computation of the canard orbits via a trivial and simple method that
        % uses a linear update.
        [ICCF_LM,ICCB_LM,IP_LM] = Func_FindIntersection(IF,IB,ICF,flipud(ICB),nC,0);
        [SCF_LM, SCB_LM, Error_LM] = Func_ComputeCanard(ICCF_LM,ICCB_LM,...
            @Func_LAV_3D,param,VAR3D,PFST,IF,IB,IP_LM,0,1);
        
        % ========================= MARKER: CODE STATUS =======================
        % Marker use to track the code status. These print information inside 
        % cmd.
        fprintf('Completed!\n')
        fprintf('CANARD - Canard Computation with Optimized Method...')
        
        % ================== CANARDS: OPTIMIZATION METHOD =====================
        % Computation of the canard orbits via an optimization method that 
        % moves the initial condition inside the IC lines for both backward and
        % forward computation to get the point matching inside the folded node 
        % shifter plane.
        [ICCF_OP,ICCB_OP,IP_OP] = Func_FindIntersection2(Geom,param,IF,IB,ICF,flipud(ICB),nC,0);
        [SCF_OP, SCB_OP, Error_OP] = Func_ComputeCanard(ICCF_OP,ICCB_OP,...
            @Func_LAV_3D,param,VAR3D,PFST,IF,IB,IP_OP,0,1);
        
        % ======================= MARKER: CODE STATUS =========================
        % Marker use to track the code status. These print information inside 
        % cmd.
        fprintf('Completed!\n')
    
        % ========================= CHECKING RESULTS ==========================
        % Inspection of the results via both the temporal visualization but
        % even the plane projection to understand how much the algorithm
        % creates of error.
                      VARPi = {'h','V'};
        fig = figure; Func_VisualizePlaneProjection3(SF,1,1);
                      Func_VisualizePlaneProjection3(SB,2,1);
                      Func_VisualizePlaneProjection3(SCF_OP,3,1);
                      Axes = Func_VisualizePlaneProjection3(SCB_OP,4,1);
                      Func_SetAxesFont(Axes,opts)
                      Func_SetGrid(Axes,opts)
                      box on
        
        % =========================== USER INTERACTION ========================
        % Checking if the user want to keep the results obtained or want to
        % change the one obtained by computing a lower number of canards.
        ANS = input('CANARD - Do you want to keep the results? [0 - No | 1 - Yes] ');
    
        if ANS == 1
            REQUEST = false;
            % ===================== CANARD: CONSTRUCTION ======================
            % Construction of the canard orbit and saving of the same inside 
            % dedicated structure that are used in further visualizations. A 
            % proper function is defined in order to handle the different 
            % informations.
            
            % ========================= HYPEPARAMETERS ========================
            % Hypeparameters to execute the code and let it be more readable.
            fieldsToMerge = {'t','V','h','s'};
            
            % ====================== MARKER: CODE STATUS ======================
            % Marker use to track the code status. These print information 
            % inside cmd.
            fprintf('CANARD - Canard Structure Construction...')
            
            % ====================== CANARD: CONSTRUCTION =====================
            % Construction of the canard orbit via a dedicated function.
            C = Func_BuildCanard(SCF_OP, SCB_OP, fieldsToMerge);
            
            % ====================== MARKER: CODE STATUS ======================
            % Marker use to track the code status. These print information 
            % inside cmd.
            fprintf('Completed!\n')
            fprintf('CANARD - Canard Classification...')
            
            % ======================= CANARD: CLASSIFICATION ==================
            % Classification of the canard orbit via the identification of how 
            % many small amplitude oscillations are performed before executing 
            % the large excursion.
            C = Func_ClassifyCanard(C,-20);
            
            % ====================== MARKER: CODE STATUS ======================
            % Marker use to track the code status. These print information 
            % inside cmd.
            fprintf('Completed!\n')
    
            % ========================= RESULTS: EXPORTATION ==================
            % Exportation of the results inside a .mat file that will be used
            % in further computations.
            save([pathCanard,fileNameCanard],'C','SCF_OP','SCB_OP')
        else
            REQUEST = true; 
        end
    end
else
    load([pathCanard,fileNameCanard])
end

%% CANARD ORBIT - TESTING
% Testing canard orbit via the forward computation of the canard retrived
% during the execution of the canard identification and retrival script.

CR = Func_TestingCanardOrbit(C,param);
CR = Func_ClassifyCanard(CR,0);

VARVisual = {'t','V'};

fig = figure; tiledlayout(2,1,'TileSpacing','tight','Padding','tight')
              nexttile(1)
              [~, p] = Func_VisualizeCanard2(fig,C,VARVisual);
              Func_CreateCanardLegend(p,C,length(C));
              nexttile(2)
              [~, p] = Func_VisualizeCanard2(fig,CR,VARVisual);
              Func_CreateCanardLegend(p,CR,length(CR));

%% CANARD - VISUALIZATION
% Visualization of the canard orbit inside the phase space and in the time
% domain. These are even visualized with additional information on the
% critical and slow manifold.

% ====================== VISUALIZATION: SETTINGS ==========================
% Definition of the visualization settings exploited inside further section
% of the code. It should be act on this section if any visualization
% hypeparameters is needed to be changed or adjusted.
opts.xlim = [FS.h*0.85 FS.h*1.05];
opts.ylim = [FS.V*1.05 FS.V*0.95];
opts.zlim = [FS.s*0.90 FS.s*1.10];

% ====================== PHASE SPACE: VISUALIZATION =======================
% Visualization of the phase space canard behaviour.
fig = figure;  [Axes,p] = Func_VisualizeCanard2(fig,C,VAR3D);
               Func_SetGrid(Axes,opts)
               Func_SetAxesFont(Axes,opts)
               Func_SetBoundaries(opts)
               Func_CreateCanardLegend(p,C,5);
               xlabel('h [$\ $]','interpreter','latex')
               ylabel('V [$mV$]','interpreter','latex')
               zlabel('s [$\ $]','interpreter','latex')

% ====================== VISUALIZATION: SETTINGS ==========================
% Definition of the visualization settings exploited inside further section
% of the code. It should be act on this section if any visualization
% hypeparameters is needed to be changed or adjusted.
opts.xlim = [0, 13]; opts.ylim = [-80, 5]; opts.zlim = [-1 1];

% ====================== TIME TRACES: VISUALIZATION =======================
% Visualization of the temporal evolution of the canard behaviour.
fig = figure;  [Axes,p] = Func_VisualizeCanard2(fig,C,{'t','V'});
               Func_SetGrid(Axes,opts)
               Func_SetAxesFont(Axes,opts)
               Func_SetBoundaries(opts)
               Func_CreateCanardLegend(p,C,5);
               xlabel('t [$ms$]','interpreter','latex')
               ylabel('V [$mV$]','interpreter','latex')