%
% Canard Identification with Brute-Force Forward Simulation
%
% ---- Note ---------------------------------------------------------------
% The algorithm identifies the missing canards between the strong canard
% and the first maximal canard reconstructed through slow manifold
% intersection. The classification is made by looking at the number of
% small amplitude oscillations activated.
%
% ---- Requirements -------------------------------------------------------
% It requires the following folder with the data made available to work
% properly. Specifically, it needs:
%
%   -   Functions:  Folder containing the functions used in the code.
%   -   Figures  :  Folder where to export figures.
%   -   Data     :  Folder storing the required data.
%
% ---- Authors ------------------------------------------------------------
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

%% PREPARATION
% Preparation of the file's workspace and MATLAB's enviroment.
close all; clear all; clc;

%% FUNCTIONS
% Add to path the function's folder where all the different functions are
% contained.
addpath('../Function/')

%% PATH
% Path of where are stored the different .mat file which are produced with
% this program and can be used to perform or not computations.

pathGeometry             = '../Data/Geometry/';
pathSlowManifold         = '../Data/SlowManifold/BVP/';
pathSlowManifoldSettings = '../Data/SlowManifold/Settings/';
path3DSimulation         = '../Data/3DSimulations/';
path5DSimulation         = '../Data/5DSimulations/';
pathOptions              = '../Data/Options/';
pathCanard               = '../Data/Canard/';

%% HYPERPARAMETERS
% Definition of variables and constants that constitutes the
% hyperparameters of the code to make it more readable and clear.

% VARIABLES - Variables structure used to visualize and store information
VAR = {'V','h','s'}; VARVisualPS = {'h','V','s'}; VARVisualTS = {'t','V'};

% DEBUG - Debug variables used to control the visualization inside the
%         comand window and with dedicated figures of the elaboration
%         state of the code.
isVisualDebug = false;

%% MODEL'S PARAMETER
% Definition of the model's parameter to avoid to define them inside the
% code and generating problem in the different simulations

% ================================ SETTINGS ===============================
% Settings used to select which kind of experiment to consider of which
% reconstruct the slow manifold.
iEXP     = input('EXP - Select Experiment: [1-5]');                     % Experiments
iCurrent = input('EXP - Select Current: [1 - Low | 2 - High]');         % Current
CTRL     = input('EXP - Select State: [Control | Stimulated]','s');     % Control/Stimulated

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('PREPARATION - Experiment Information Retrival...')

% ============================= EXPERIMENT ================================
% Experiment information extraction. These will be then used to tune where
% to reconstruct the Slow Manifold in order to explain the dynamical system
% model behaviour.
fileNameSimulation = sprintf('LAV_3D_Simulation_Exp%i.mat',iEXP);
load([path3DSimulation,fileNameSimulation]);
EXP        = eval(['EXP',num2str(iEXP),';']);
param      = eval(['EXP.',CTRL,'.param;']);
param.r    = eval(['EXP.',CTRL,'.rAVG(iCurrent)*EXP.',CTRL,'.KHCN;']);
param.w    = eval(['EXP.',CTRL,'.wAVG(iCurrent)*EXP.',CTRL,'.KM;']);
param.IApp = eval(['EXP.',CTRL,'.IApp(iCurrent);']);
S          = eval(['EXP.',CTRL,'.S{iCurrent};']);

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('PREPARATION - File Name Construction...')

% ========================= FILE NAME: DEFINITION =========================
% Definition of the main file names that can be used in further section of
% the code to improve the code readibility.
fileNameGeometry     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Geometry'];
fileNameSettings     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_BVPSettings'];
fileNameSlowManifold = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_SlowManifold'];
fileNameOptions      = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Options'];
fileNameCanard       = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Canard'];
fileNameCanardUpdate = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_CanardUpdated'];

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('PREPARATION - Visualization Setting Loading...')

% ========================= VISUALIZATION SETTINGS ========================
% Settings exploited to execute correctly the further proposed
% visualizations. These settings will be exploited in the further sections
% of the program.
load([pathOptions,fileNameOptions]);
opts.Font     = 'Times New Roman';
opts.FontSize = 12;

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('GEOMETRY - Loading...')

% ======================= LOADING: MODEL'S GEOMETRY =======================
% Loading of the model's geometry inside the matlab workspace to be used in
% further computation and eventual visualizations.
load([pathGeometry,fileNameGeometry])

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('SLOW MANIFOLD - Loading...')

% ======================= LOADING: COMPUTED SLOW MANIFOLD =================
% Loading of the slow manifold computed in another .m file produced via
% Matlab. This is foundamental to perform the analyses proposed further.
load([pathSlowManifold,fileNameSlowManifold])

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')
fprintf('CANARD - Loading...')

% ======================= LAODING: COMPUTED CANARD ORBITS =================
% Loading of the computed canard orbits inside the workspace of matlab to
% be used inside the further computations.
load([pathCanard,fileNameCanard]), nC = length(C);

% ========================= MARKER: CODE STATUS ===========================
% Marker use to track the code status. These print information inside cmd.
fprintf('Completed!\n')

%% FORWARD AND BACKWARD IC BOUNDARIES - CANARD
% Identification of the initial condition associated to the canard orbit
% with lower amount of small amplitude oscillations. This is necessary to
% find the boundary where to search the non-identified canard orbit.

nV = length(VAR);
for iV = 1:1:nV
    SPCCFC.(VAR{iV}) = C{1}.(VAR{iV})(1);
    SPCCBC.(VAR{iV}) = C{1}.(VAR{iV})(end);
end

%% FORWARD AND BACKWARD IC BOUNDARIES - CANDIDATE IC
% Definition of new candidate segments where to search for the
% non-identified canard orbit. These new segments are extensions of the
% initial conditions segments that we have initially used to reconstruct
% the model's slow manifold.

% =============================== ACCURACY ================================
% Accuracy with which this extension segments will be reconstructed to
% infer information about the new testing initial conditions.
nP = 1000; 

% ========================= FORWARD: INFORMATION ==========================
% Forward extension segment reconstruction. The rationale used in this case
% is the same that was exploited previously. In this case first we retrive
% the minimum voltage's value associated to the initial condition of the
% first identified canard orbit (Vm) and then we create the segment where
% to investigate the presence of non-identified and recognized canard
% orbits.

% SETTINGS - Settings and information used in the following lines of code
Vm = FS.V; h = unique(LF.h); c = unique(LF.C);

% RECONSTRUCTION - Reconstruction of the extension segment non-filtered
CCLF.V = linspace(SPCCFC.V,Vm,nP)'; CCLF.h = h*ones(nP,1); CCLF.s = Geom.C0s(CCLF.V, CCLF.h);
if c, CCLF.C = true(nP,1); else, CCLF.C = false(nP,1); end

% RECONSTRUCTION - Filtering of the previously proposed extension segment
index = CCLF.s >= EQ.s; CCLF = Func_Crop(CCLF,index); CCLF = Func_EnhanceLine(CCLF,Geom,nP,'Linear',1);

%% CANARD - RECONSTRUCTION
% Reconstruction of the canard orbit via the forward propagation only. The
% canard orbit, as above stated, are identified as those segments from
% where the system changes the overall number of SAO produced. These are
% the last trajectories presenting a given number of SAO.

% ======================== SOLUTION SPACE: CROPPING =======================
% Cropping of the solution space via the identification of the initial
% condition associated to the first canard orbit identified via the
% intersection of the two slow manifold.
I = find(CCLF.s < C{1}.s(1),1,'first');

% ============================= ODE SETTINGS ==============================
% Settings used inside the ODE solver to solve the system's of ODE.
T = [0 100]; nIC = length(CCLF.V); param.fC0h = Geom.C0h;
options = odeset('RelTol',1e-6, ...
                 'AbsTol',1e-6, ...
                 'Events',@(t,q)Func_EventODECriticalManifold(t,q,param));

% =========================== PRIMARY SOLUTION ============================
% First solution of the ODE system to retrive those information to
% initialize those variables handled inside the scanning cycle.
IC = [CCLF.V(nIC);CCLF.h(nIC);CCLF.s(nIC)]; 
S = Func_Simulate(@Func_LAV_3D,T,IC,VAR,param,options);
nP = sum(islocalmax(S.V)); nC = 0; Stemp = S;

% ============================ RETRIVAL: INFORMATION ======================
% Retrival of necessary information as the number of already identified
% canard orbit.
nCanard = length(C);

% =========================== CANARD: RECONSTRUCTION ======================
% Reconstruction of the canard orbit via the dedicated algorithm. The
% dynamical system is solved starting from a given initial condition. If
% the number of SAO associated to that solution changes from the previous
% trajectory then the previous computed solution is saved as a canard
% orbit.
for i = nIC-1:-1:I

    IC = [CCLF.V(i);CCLF.h(i);CCLF.s(i)];
    S = Func_Simulate(@Func_LAV_3D,T,IC,VAR,param,options);
    nC = sum(islocalmax(S.V));

    if not(nP == nC)
        fprintf('ELAB - Computed Canard - nC = %i\n',nP)
        CU{nP+1}.nSAO = nP;   CU{nP+1}.Leg = ['$\xi_{',num2str(nP),'}$'];
        CU{nP+1}.t = Stemp.t; CU{nP+1}.h = Stemp.h;
        CU{nP+1}.s = Stemp.s; CU{nP+1}.V = Stemp.V;
    end

    nP = nC; Stemp = S;
end

% ============================= CANARD: FILLING ===========================
% Filling of the canard orbit computed at the previous step with the one
% already computed via the intersection of the slow manifold method.
nC = length(CU);

for i = 1:1:nCanard
    CU{nC+i}.nSAO = C{i}.nSAO;   CU{nC+i}.Leg = ['$\xi_{',C{i}.nSAO,'}$'];
    CU{nC+i}.t = C{i}.t;         CU{nC+i}.h = C{i}.h;
    CU{nC+i}.s = C{i}.s;         CU{nC+i}.V = C{i}.V;
end

C = CU;

%% CANARD - VISUALIZATION
% Checking the results' reliability.

fig = figure; subplot(1,2,1), Func_VisualizeCanard2(fig,C,VARVisualPS);
              subplot(1,2,2), Func_VisualizeCanard2(fig,C,VARVisualTS);

%% DATA - EXPORT
% Exportation of the data inside a dedicated .mat file inside the path
% initially declared. The information exported consists just of the canard
% orbits.

ANS = input('REQUEST - Do you want to export the file? [0 - No | 1 - Yes]');

if ANS == 1
    fprintf('DATA - Export ...')
    save([pathCanard,fileNameCanardUpdate],'C');
    fprintf('Completed!\n')
end