function CT = Func_SelectBestApproximatedCanard(S,C)
%
%   CT = Func_SelectBestApproximatedCanard(S,C)
%
%   Function that select the best approximated canard solution betweeen the
%   one specified inside the S structure passed as input to this routine.
%   The chosen canard solution are the one missing inside the canard vector
%   C. The stored inside the vector CT are sorted depending on the number
%   of SAO that each identified solution present.
%
%   @param S    :   Approximated canard orbit.
%   @param C    :   Canard orbit identified via the intersection between
%                   the attracting and repelling slow manifold.
%
%   @output CT  :   Canard orbit cell array where the order of the cell
%                   array is function of the number of SAO produced from
%                   each canard solution.
%
% PhD Student Martin Matteo
% Department of Information Engineering
% University of Padova

% ========================= INFORMATION: RETRIVAL =========================
% Retrival of the information necessary in further section of the code. The
% canard orbit with the minimum number of SAO is identified in order to
% automatically detect the one that needed to be retrived from the
% approximated computation.
nC = length(C); m = nC;
for iC = 1:1:nC, if C{iC}.nSAO < m && C{iC}.nSAO ~= 0, m = C{iC}.nSAO; end, end

% ============================= CANARD: UPDATE ============================
% Update of the canard cell array with the one identified via the
% approximation. The new element are placed at the end of the cell array.
% The approximation solution selected is the one that minimizes the
% conjunction error.
nS = length(S); iSelect = [];
for iSAO = 1:1:m-1
    mE = 100;

    for iS = 1:1:nS
        if S{iS}.nSAO == iSAO
            if mE > S{iS}.e, mE = S{iS}.e; iSelect = iS; end
        end
    end

    C{nC+iSAO}.V = S{iSelect}.S.V; C{nC+iSAO}.h = S{iSelect}.S.h; 
    C{nC+iSAO}.s = S{iSelect}.S.s; C{nC+iSAO}.t = S{iSelect}.S.t;
    C{nC+iSAO}.nSAO = iSAO; C{nC+iSAO}.Leg = sprintf('$\\xi_{%i}$',iSAO);
end

% ============================ INFORMATION: RETRIVAL ======================
% Generation of a vector containing in each element the number of SAO
% produced in each identified canard orbit both computed with approximation
% or not.
nC = length(C); CT = cell(1,nC); nSAO = zeros(nC,1);
for iC = 1:1:nC, nSAO(iC) = C{iC}.nSAO; end

% =========================== CANARD: SORTING =============================
% Creation of the sorted canard structure where each cell of the cell
% array vector contains a canard orbit. The number of SAO in successive
% cell of the array increases. Moreover, each of the presented canard orbit
% are classified as intersection or approximation depending on the
% computational technique exploited.
[nSAO, iCanardSAO] = sort(nSAO,'ascend');
for iC = 1:1:nC
    CT{iC} = C{iCanardSAO(iC)}; 

    if nSAO(iC) < m, CT{iC}.Meth = 'Approximation';
    else,            CT{iC}.Meth = 'Intersection';
    end
end

end