%
% Geometry
%
% ---- Note ---------------------------------------------------------------
% This script produces a .mat file containing the geometry of the model.
% It can be used to independently calculate the whole model geometry.
%
% ---- Requirements -------------------------------------------------------
% It requires the following folder with the data made available to work
% properly. Specifically, it needs:
%
%   -   Functions:  Folder containing the functions used in the code.
%   -   Figures  :  Folder where to export figures.
%   -   Data     :  Folder storing the required data.
%
% ---- Authors ------------------------------------------------------------
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

%% PREPARATION
% Preparation of the file's workspace and MATLAB's enviroment.
close all; clear all; clc;

%% FUNCTIONS
% Add to path the function's folder where all the different functions are
% contained.
addpath('../Function/')

%% PATH
% Path of where are stored the different .mat file which are produced with
% this program and can be used to perform or not computations.

pathGeometry             = '../Data/Geometry/';
pathSlowManifold         = '../Data/SlowManifold/BVP/';
pathSlowManifoldSettings = '../Data/SlowManifold/Settings/';
path3DSimulation         = '../Data/3DSimulations/';
path5DSimulation         = '../Data/5DSimulations/';
pathOptions              = '../Data/Options/';

%% MODEL'S PARAMETER
% Definition of the model's parameter to avoid to define them inside the
% code and generating problem in the different simulations

% ================================ SETTINGS ===============================
% Settings used to select which kind of experiment to consider of which
% reconstruct the slow manifold.
iEXP     = input('EXP - Which experiment would you consider? [1-5]');                 % Experiment
iCurrent = input('EXP - Which stimulation current to investigate? [1-Low | 2-High]'); % Current
CTRL     = input('EXP - Which condition? [Control | Stimulated]','s');                % Control/Stimulated

% ============================= EXPERIMENT ================================
% Experiment information extraction. These will be then used to tune where
% to reconstruct the Slow Manifold in order to explain the dynamical system
% model behaviour.
fileNameSimulation = sprintf('LAV_3D_Simulation_Exp%i.mat',iEXP);
load([path3DSimulation,fileNameSimulation]);
EXP        = eval(['EXP',num2str(iEXP),';']);
param      = eval(['EXP.',CTRL,'.param;']);
param.r    = eval(['EXP.',CTRL,'.rAVG(iCurrent)*EXP.',CTRL,'.KHCN;']);
param.w    = eval(['EXP.',CTRL,'.wAVG(iCurrent)*EXP.',CTRL,'.KM;']);
param.IApp = eval(['EXP.',CTRL,'.IApp(iCurrent);']);
S          = eval(['EXP.',CTRL,'.S{iCurrent};']);

% ============================== MARKER ===================================
% Request to the user to check the equivalent value of the activation
% variables r and w used inside the considered iteration.
REQUEST = input(sprintf('REQUEST - Would you continue? (r = %f, w = %f) [0-No | 1-Yes]',param.r,param.w));
if REQUEST == 0, error('ERROR - Experiment''s values of r and w invalid!'), end

% ========================= FILE NAME: DEFINITION =========================
% Definition of the main file names that can be used in further section of
% the code to improve the code readibility.
fileNameGeometry     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Geometry'];
fileNameSettings     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_BVPSettings'];
fileNameSlowManifold = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_SlowManifold'];
fileNameOptions      = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Options'];

% ========================= VISUALIZATION SETTINGS ========================
% Settings exploited to execute correctly the further proposed
% visualizations. These settings will be exploited in the further sections
% of the program.
isVisual = false;
if input('OPTS - Would you like to redefine the option structure? [0 - No | 1 - Yes]') == 1
    opts = Func_DeafultVisualizationSettings();
    opts.width = 8.25;      opts.height = 8.25;
else
    load([pathOptions,fileNameOptions]);
end

% ======================== EXPERIMENT: INFORMATION ========================
% Extraction of information that will be used to reconstruct the slow
% manifold in the further section of the code.
T = 350;                                % Cropping time
S = Func_CropTime(S,T);                 % Cropping simulation structure
eval(['EXP.',CTRL,'.S{iCurrent} = S;']) % Cropping experiment structure
[~,I] = min(S.V); mSh = S.h(I);
I = find(S.V>0);  mSs = mean(S.s(I));

%% MODEL'S VARIABLE
% Definition of the model's variable that will be inspected to perform all
% the possible analysis over the considered model.

VAR3D = {'h','V','s'}; ACT3D = {'h','s'};

%% MODEL'S BOUNDARIES
% Boundaries necessary to perform all the different simulations and
% visualizations. Act on this structure involves changes respectevely
% inside visualization and boundaries of computed curves or surfaces.

B.t = [0 50]; B.V = [-83.8 80]; B.h = [0 1]; B.s = [0 1];

%% MODEL GEOMETRY - COMPUTATION
% Computation of the model geometry given a specific set of parameters.
% After having derived the C0, L, NSW, EQ and FS of relevant of the system
% the same are converted into objects that can be successively visualized.

if input('GEOM - Do you want to recompute the model''s geometry? [0 - No | 1 - Yes]') == 1
    % =============================== SETTINGS ============================
    % Settings for geometrical computation of model properties.
    opts.C0.V = [-83.8 80];  opts.C0.h  = [0 1];        
    opts.C0.s = [0 1];       opts.nP   = 3000;
    opts.L.V  = [-80 -37.6]; opts.NSW.V = [-83.8 -37.6];
    
    % ========================= GEOMETRY COMPUTATION ======================
    % Derivation of the dynamical system geometry through the parameter
    % structure passed as input to the geomtry derivation function.
    [Geom,EQ,FS] = Func_LAV3DGeometry(param,true);
    
    % ======================== GEOMETRY CONVERSION ========================
    % Conversion of the previous geometry structure into matlab vectors and
    % matrices that can be visualzied through dedicated functions.
    C0s = Func_ComputeGeomC0(Geom.C0s,Geom.dVf,opts,'h');
    L   = Func_ComputeGeomL(Geom.L,Geom.C0s,opts);
    NSW = Func_ComputeGeomNSW(Geom.NSW,Geom.C0s,opts);

    % ============================ SAVAGE =================================
    % Exportation of the geometry file inside the dedicated folder
    % specified inside the initial section of this file.
    save([pathGeometry,fileNameGeometry],'Geom','EQ','FS','C0s','L','NSW')
    save([pathOptions,fileNameOptions],'opts')
else
    % =========================== LOADING =================================
    % Loading of the geometry file specified inside the file name at the
    % beginning of this script.
    load([pathGeometry,fileNameGeometry])
    load([pathOptions,fileNameOptions])
end