%
% Slow Manifold Reconstruction - EXP1 IApp 250
%
% ---- Note ---------------------------------------------------------------
% This script reconstruct an approximation of the slow manifold Sr and Sa
% close to the folded node to explain the small amplitude oscillations
% observed during the trajectory evolution. This script only reconstruct
% the slow manifolds. It does not reproduce the figure presented in the
% paper. To create those visualizations run the visualization algorithms.
%
% ---- Requirements -------------------------------------------------------
% It requires the following folder with the data made available to work
% properly. Specifically, it needs:
%
%   -   Functions:  Folder containing the functions used in the code.
%   -   Figures  :  Folder where to export figures.
%   -   Data     :  Folder storing the required data.
%
% ---- Authors ------------------------------------------------------------
% PhD Student Martin Matteo
% University of Padova
% Department of Information Engineering

%% PREPARATION
% Preparation of the file's workspace and MATLAB's enviroment.
close all; clear all; clc;

%% FUNCTIONS
% Add to path the function's folder where all the different functions are
% contained.
addpath('../Function/')

%% PATH
% Path of where are stored the different .mat file which are produced with
% this program and can be used to perform or not computations.

pathGeometry             = '../Data/Geometry/';
pathSlowManifold         = '../Data/SlowManifold/BVP/';
pathSlowManifoldSettings = '../Data/SlowManifold/Settings/';
path3DSimulation         = '../Data/3DSimulations/';
path5DSimulation         = '../Data/5DSimulations/';
pathOptions              = '../Data/Options/';

%% MODEL'S PARAMETER
% Definition of the model's parameter to avoid to define them inside the
% code and generating problem in the different simulations

% ================================ SETTINGS ===============================
% Settings used to select which kind of experiment to consider of which
% reconstruct the slow manifold.
iEXP     = 1;               % Experiment
iCurrent = 1;               % Current
CTRL     = 'Stimulated';    % Control/Stimulated

% ============================= EXPERIMENT ================================
% Experiment information extraction. These will be then used to tune where
% to reconstruct the Slow Manifold in order to explain the dynamical system
% model behaviour.
fileNameSimulation = sprintf('LAV_3D_Simulation_Exp%i.mat',iEXP);
load([path3DSimulation,fileNameSimulation]);
EXP        = eval(['EXP',num2str(iEXP),';']);
param      = eval(['EXP.',CTRL,'.param;']);
param.r    = eval(['EXP.',CTRL,'.rAVG(iCurrent)*EXP.',CTRL,'.KHCN;']);
param.w    = eval(['EXP.',CTRL,'.wAVG(iCurrent)*EXP.',CTRL,'.KM;']);
param.IApp = eval(['EXP.',CTRL,'.IApp(iCurrent);']);
S          = eval(['EXP.',CTRL,'.S{iCurrent};']);

% ========================= FILE NAME: DEFINITION =========================
% Definition of the main file names that can be used in further section of
% the code to improve the code readibility.
fileNameGeometry     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Geometry'];
fileNameSettings     = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_BVPSettings'];
fileNameSlowManifold = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_SlowManifold'];
fileNameOptions      = ['EXP',num2str(iEXP),'_IApp',...
                        num2str(param.IApp),'_',CTRL,'_Options'];

% ========================= VISUALIZATION SETTINGS ========================
% Settings exploited to execute correctly the further proposed
% visualizations. These settings will be exploited in the further sections
% of the program.
isVisual = false;
if input('OPTS - Would you like to redefine the option structure? [0 - No | 1 - Yes]') == 1
    opts = Func_DeafultVisualizationSettings();
    opts.width = 8.25;      opts.height = 8.25;
else
    load([pathOptions,fileNameOptions]);
end

% ======================== EXPERIMENT: INFORMATION ========================
% Extraction of information that will be used to reconstruct the slow
% manifold in the further section of the code.
T = 350;                                % Cropping time
S = Func_CropTime(S,T);                 % Cropping simulation structure
eval(['EXP.',CTRL,'.S{iCurrent} = S;']) % Cropping experiment structure
[~,I] = min(S.V); mSh = S.h(I);
I = find(S.V>0);  mSs = mean(S.s(I));

% ======================== SLOW MANIFOLD: RESOLUTION ======================
% Slow manifold resolution. Definition of the number of simulations that
% are required to reconstruct the previous geometrical structure.
nSLOW = 1000;

%% MODEL'S VARIABLE
% Definition of the model's variable that will be inspected to perform all
% the possible analysis over the considered model.

VAR3D = {'h','V','s'}; ACT3D = {'h','s'};

%% MODEL'S BOUNDARIES
% Boundaries necessary to perform all the different simulations and
% visualizations. Act on this structure involves changes respectevely
% inside visualization and boundaries of computed curves or surfaces.

B.t = [0 50]; B.V = [-83.8 80]; B.h = [0 1]; B.s = [0 1];

%% MODEL GEOMETRY - COMPUTATION
% Computation of the model geometry given a specific set of parameters.
% After having derived the C0, L, NSW, EQ and FS of relevant of the system
% the same are converted into objects that can be successively visualized.

if input('GEOM - Do you want to recompute the model''s geometry? [0 - No | 1 - Yes]') == 1
    % =============================== SETTINGS ============================
    % Settings for geometrical computation of model properties.
    opts.C0.V = [-83.8 80];  opts.C0.h  = [0 1];        
    opts.C0.s = [0 1];       opts.nP   = 3000;
    opts.L.V  = [-80 -37.6]; opts.NSW.V = [-83.8 -37.6];
    
    % ========================= GEOMETRY COMPUTATION ======================
    % Derivation of the dynamical system geometry through the parameter
    % structure passed as input to the geomtry derivation function.
    [Geom,EQ,FS] = Func_LAV3DGeometry(param,true);
    
    % ======================== GEOMETRY CONVERSION ========================
    % Conversion of the previous geometry structure into matlab vectors and
    % matrices that can be visualzied through dedicated functions.
    C0s = Func_ComputeGeomC0(Geom.C0s,Geom.dVf,opts,'h');
    L   = Func_ComputeGeomL(Geom.L,Geom.C0s,opts);
    NSW = Func_ComputeGeomNSW(Geom.NSW,Geom.C0s,opts);

    % ============================ SAVAGE =================================
    % Exportation of the geometry file inside the dedicated folder
    % specified inside the initial section of this file.
    save([pathGeometry,fileNameGeometry],'Geom','EQ','FS','C0s','L','NSW')
    save([pathOptions,fileNameOptions],'opts')
else
    % =========================== LOADING =================================
    % Loading of the geometry file specified inside the file name at the
    % beginning of this script.
    load([pathGeometry,fileNameGeometry])
    load([pathOptions,fileNameOptions])
end

% =============================== OPTIONS =================================
% Visualization options.
opts.Font = 'Times New Roman'; opts.FontSize = 12;

%% MODEL GEOMETRY - VISUALIZATION
% Visualization of the model geometry computed at the previous step with
% all the additional necessary informations.

% ============================= VISUALIZATION =============================
fig = figure(); hold on
                Func_VisualizeGeomC0(C0s);
                Func_VisualizeGeomL(L);
                Axes = Func_VisualizeGeomNSW(NSW);
                Func_VisualizeEQ(EQ);
                Func_VisualizeFS(FS);
                hold off
                Func_SetGrid(Axes,opts)
                Func_SetAxesFont(Axes,opts)
                Func_SetBoundaries(opts)
                view(45,45)
                xlabel('$h$ [$\ $]','interpreter','latex')
                ylabel('$V$ [$mV$]','interpreter','latex')
                zlabel('$s$ [$\ $]','interpreter','latex')
                if isVisual, Func_SaveFigure(fig,sprintf('LAV_3D_C0_EXP%i',iEXP),opts); end

%% SIGMA FN - COMPUTATION
% Computation of the plane orthogonal to the flux in correspondence of the
% folded node. This is then visualized inside a dedicated image where even
% information about the system geometry are displayed.

% ============================ VECTOR COMPUTATION =========================
PFS      = Func_StructToVect(FS,VAR3D);
PEQ      = Func_StructToVect(EQ,VAR3D);
n        = [0; 0; 1];
T        = [0; 0; 5e-4];
PFST     = PFS - T;
SigmaFNT = Func_ComputePlane(n,PFST);
SigmaFN  = Func_ComputePlane(n,PFS);
SigmaEQ  = Func_ComputePlane(n,PEQ);

%% SIGMA FN - VISUALIZATION
% Visualization of the plane Sigma previously computed together with the
% geomtry of the model.

% ============================= VISUALIZATION =============================
fig = figure(); Func_VisualizeGeomC0(C0s);
                Func_VisualizeGeomL(L);
                Axes = Func_VisualizeGeomNSW(NSW);
                Func_VisualizeEQ(EQ);
                Func_VisualizeFS(FS);
                Func_VisualizePlane2(SigmaEQ,opts)
                Func_VisualizePlane2(SigmaFNT,opts)
                Func_SetGrid(Axes,opts)
                Func_SetAxesFont(Axes,opts)
                Func_SetBoundaries(opts)
                view(45,15)
                xlabel('$h$ [$\ $]','interpreter','latex')
                ylabel('$V$ [$mV$]','interpreter','latex')
                zlabel('$s$ [$\ $]','interpreter','latex')
                if isVisual, Func_SaveFigure(fig,sprintf('LAV_3D_C0_Planes_EXP%i',iEXP),opts); end

%% SLOW MANIFOLD - COMPUTATIONS
% Slow manifold computation process. The next procedure is split into two
% steps. The first require to identify for which initial condition solve
% the forward and backward boundary value problems.
%
% STEP 1 - Initial Condition Optimizaiton
%   Identification of the initial conditions for which the code should be
%   executed and the trajectory studied. The number of considered planes
%   are two. 
%       - Plane 1: It intersects the critical manifold by fixing h.
%                  It acts as a vertical section.
%       - Plane 2: It intersects the critical manifold by fixing s.
%                  It acts as a horizontal section.
%   The intersections create two lines with h and s respectively fixed.
%   These are the points from where the forward and backward BVP are
%   solved.
%
% STEP 2 - BVP Solution
%   Solution of both backward and forward BVP using as ICs the points
%   contained inside the two curves which intersects the critical manifold
%   horizontally and vertically.

if input('BVP - Do you want to solve again forward and backward BVPs? [0 - No | 1 - Yes]') == 1
    
    fprintf('BVP - STEP 1 - Options forward ...')
    % ============================= OPTIONS FORWARD =======================
    % Definition of the integration options for the forward propagation
    % algorithm to reconstruct the attracting slow manifold.
    param.VAR = {'V','h','s'}; param.Sigma = SigmaFNT; 
    param.TL  = 0;             param.TH    = 150;
    param.D   = [0; -1];
    
    % ================================ PLANES =============================
    % Vertical plane which h coordinate is fixed to the one associated to 
    % EQ. Correction of the plane intersetion respect to the specified h to
    % a value for which L has been already computed.
    SigmaFSV2 = Func_ComputePlane([1;0;0],[mSh;-80;0.3]);
    PVF       = Func_ComputeIntersectionL(SigmaFSV2,L);
    SigmaFSV1 = Func_ComputePlane([1;0;0],PVF);
    
    % ============================== INTERSECTIONS ========================
    % Computation of the intersections between the planes and the C0.
    PF = Func_VectToStruct(Func_ComputeIntersectionL(SigmaFSV1,L),VAR3D);
    LF = Func_ComputeIntersectionC0(SigmaFSV1,Geom.C0s,Geom.dVf,opts);
    
    % ============================ CURVES CORRECTIONS =====================
    % Cropping of the vertical line used as initial conditions value to
    % reconstruct the slow manifold close to the folded singularity of the
    % system. Once cropped such that, the IC would belong to the stable
    % branch, the amplitude s would be always contained inside the interval 
    % [FS.s 1].
    index = LF.V < PF.V;      LF = Func_Crop(LF,index);
    index = LF.s < 0.75;      LF = Func_Crop(LF,index);
    index = LF.s > FS.s;      LF = Func_Crop(LF,index);
    index = LF.C == 0;        LF = Func_Crop(LF,index);
    
    % ============================= CURVE SHIFTS ==========================
    % Shift of the curve through the information retrived from the 
    % experiment performed and from which simulations needed to be run. The
    % whole set of initial conditions that are found are then validated 
    % evaluating if they produce or not any oscillation during the 
    % evolution of the dynamical system.
    LF = Func_EnhanceLine(LF,Geom,nSLOW,'Log',10);
    LF = Func_DetectSimulationBoundary(LF,@Func_LAV_3D,param,@Func_EventODEAP,'F',false);
    LF = Func_EnhanceLine(LF,Geom,nSLOW,'Log',10);
    
    % ============================= FORWARD MINIMUM PLANE =================
    % Computation of the plane which presents the lowest location for which
    % each trajectory that will be analysed to reconstruct the slow 
    % manifold would present for sure intersections.
    ICF(:,1) = LF.V; ICF(:,2) = LF.h; ICF(:,3) = LF.s;
    [SigmaDW, PDW] = Func_ComputeMinimumPlane(@Func_LAV_3D,param,ICF,0,0,opts);
    DW = Func_VectToStruct(PDW,VAR3D);
    
    % ============================ LINE ENHANCEMENT =======================
    % Enhancement of the line previously computed. The reason why a new 
    % step of line enhancment is needed is beacuse the first step just give
    % some prior on where looking to instead the second identifies the 
    % right boundaries for which derive the projection until the minimum 
    % plane.
    param.Sigma = SigmaDW;
    LF = Func_DetectSimulationBoundary(LF,@Func_LAV_3D,param,@Func_EventODEAP,'F',false);
    LF = Func_EnhanceLine(LF,Geom,nSLOW,'Log',10);

    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
    fprintf('BVP - STEP 1 - Options backward ...')

    % ============================= OPTIONS BACKWARD ======================
    % Definition of the integration options for the backwards propagation
    % algorithm to reconstruct the unstable slow manifold.
    param.VAR          = {'V','h','s'}; param.Sigma = SigmaFNT; 
    param.TL           = 0;             param.TH    = -100;      
    param.NormBoundary = 500;
    
    % ================================ PLANES =============================
    % Horizontal plane which intersects C0 in a region in which AP 
    % intersects when produced. The value of s for which the plane is 
    % computed is associated to the minimum s value of AP simulations 
    % computed one after the other.
    AP        = Func_AP(param,VAR3D);
    SigmaFSH2 = Func_ComputePlane([0;0;1],[0.3;-80;mSs]);
    PHB       = Func_ComputeIntersectionL(SigmaFSH2,L); 
    SigmaFSH1 = Func_ComputePlane([0;0;1],PHB);
    
    % ============================== INTERSECTIONS ========================
    % Computation of the intersections between the planes and the C0.
    PB = Func_VectToStruct(Func_ComputeIntersectionL(SigmaFSV1,L),VAR3D);
    LB = Func_ComputeIntersectionC0(SigmaFSH1,Geom.C0h,Geom.dVf,opts);
    
    % ============================ CURVES CORRECTIONS =====================
    % Cropping of the vertical and the horizontal curves that intersects 
    % the vertical and horizontal planes with C0 such that the IC would be 
    % located always inside locations that are attracted towards the FN.
    index = LB.C == 0;       LB = Func_Crop(LB,index);
    index = LB.V > EQ.V;     LB = Func_Crop(LB,index);
    index = LB.h < AP.h;     LB = Func_Crop(LB,index);
    LB = Func_DetectSimulationBoundary(LB,@Func_LAV_3D,param,@Func_EventODENormMax,'B',false);
    LB = Func_EnhanceLine(LB,Geom,nSLOW,'Log',0.5);
    
    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
    fprintf('BVP - STEP 2 - Forward BVPs ...')

    % ========================= SLOW MANIFOLD: FORWARD ====================
    % Forward solution of the BVP to reconstruct the stable attracting
    % submanifold while relaxing the singular limit constraint.
    opts.xlim = [0 0.2]; opts.ylim = [-90 10]; opts.zlim = [0 1];
    
    % ========================= SETTINGS FORWARDS =========================
    % Settings for the solution of the two point boundary value problem. 
    % This is not solved through a bvp4 or bvp5 function of Matlab but 
    % approximated with the Event handler of the ODE solver which crop the 
    % trajectory when the first zero crosses occur.

    % - ICs
    clear ICF, nP = length(LF.V); 
    ICF(:,1) = LF.V; ICF(:,2) = LF.h; ICF(:,3) = LF.s;
    % - Simulation
    param.VAR = {'V','h','s'}; param.TL = 0; param.TH = 100;
    param.SigmaDW = SigmaDW; param.SigmaFN = SigmaFNT; 
    param.isT = [0; 1];
    param.options = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',@Func_EventODEDouble);
    
    % ============================== VISUALIZATION ========================
    % Opening of a new figure where will be visualized the forwards 
    % integration.
    fig = figure(); Func_VisualizeGeomC0(C0s);
                    Axes = Func_VisualizeGeomL(L);
                    Func_VisualizeGeomNSW(NSW);
                    Func_VisualizePlane2(SigmaDW,opts);
                    Func_VisualizeIntersectionL(LF);
                    Func_SetGrid(Axes,opts);
                    Func_SetAxesFont(Axes,opts);
                    Func_SetBoundaries(opts);
                    view(-30,15)
                    xlabel('$h$ [$\ $]','interpreter','latex')
                    ylabel('$V$ [$mV$]','interpreter','latex')
                    zlabel('$s$ [$\ $]','interpreter','latex')
    
    % ====================== FORWARDS INTEGRATION =========================
    % Forward integration step of the slow manifold approximation algorithm
    % The algorithm visualize over the previously open figure the 
    % simulation computed.
    SF = Func_BVP(ICF,@Func_LAV_3D,param,1,1,VAR3D);
    
    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
    fprintf('BVP - STEP 2 - Backward BVPs ...')

    % =========================== SETTINGS BACKWARD =======================
    % Settings for the solution of the two point boundary value problem. 
    % This is not solved through a bvp4 or bvp5 function of Matlab but 
    % approximated with the Event handler of the ODE solver which crop the 
    % trajectory when the first zero crosses occur.

    % - ICs
    clear ICB, nP = length(LB.V); 
    ICB(:,1) = LB.V; ICB(:,2) = LB.h; ICB(:,3) = LB.s; ICB = flipud(ICB);
    % - Simulation
    param.VAR = {'V','h','s'}; param.TL = 0; param.TH = -100;
    param.SigmaDW = SigmaDW; param.SigmaFN = SigmaFNT; 
    param.isT = [1; 0];
    param.options = odeset('RelTol',1e-12,'AbsTol',1e-12,'Events',@Func_EventODEDouble);
    
    % ============================== VISUALIZATION ========================
    % Opening of a new figure where will be visualized the backward 
    % integration.
    fig = figure(); Func_VisualizeGeomC0(C0s);
                    Axes = Func_VisualizeGeomL(L);
                    Func_VisualizeGeomNSW(NSW);
                    Func_VisualizePlane2(SigmaFNT,opts);
                    Func_VisualizeIntersectionL(LB);
                    Func_SetGrid(Axes,opts);
                    Func_SetAxesFont(Axes,opts);
                    Func_SetBoundaries(opts);
                    view(0,15)
                    xlabel('$h$ [$\ $]','interpreter','latex')
                    ylabel('$V$ [$mV$]','interpreter','latex')
                    zlabel('$s$ [$\ $]','interpreter','latex')
    
    % ====================== BACKWARD INTEGRATION =========================
    % Backward integration step for the slow manifold approximation 
    % algorithm. As previously already said, this visualize the simulation 
    % performed inside a given opened figure that it is computed.
    SB = Func_BVP(ICB,@Func_LAV_3D,param,1,1,VAR3D);

    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
    fprintf('BVP - Fine Tuning... ')

    % =========================== MANUAL FINE TUNING ======================
    % Manual fine tuning of the result obtined via the computations
    % previously executed.
    SF = Func_ManualFineTuning(SF,opts);
    SB = Func_ManualFineTuning(SB,opts);

    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
    fprintf('BVP - Step 3 - Saving ...')

    % =============================== SAVAGE ==============================
    % Saving of the results inside a dedicated .mat file.
    save([pathSlowManifold,fileNameSlowManifold],'SF','SB','LF','LB','FS','PFST','DW','PDW','SigmaFNT','SigmaDW')

    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
else
    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf('BVP - Loading data ...')

    % =============================== LOADING =============================
    % Loading of the file which contains the foundamental structure to
    % execute further reported and proposed computations.
    load([pathSlowManifold,fileNameSlowManifold])

    % ================================ MARKER =============================
    % Function to adversite the user on where the computation is located.
    fprintf(' Completed!\n')
end

%% SLOW MANIFOLD - VISUALIZATION: MANIFOLDS
% Visualization of the slow manifold attracting and repelling computed at
% the previous step via the set up boundary value problem of the considered
% case.

opts.xlim = [FS.h*0.85 FS.h*1.05];
opts.ylim = [FS.V*1.05 FS.V*0.95];
opts.zlim = [FS.s*0.95 FS.s*1.05];

fig = figure(); Func_VisualizeSlowManifold2(SB,'R',VAR3D,1);
                Func_VisualizeSlowManifold2(SF,'A',VAR3D,1);
                view(-75,15), Axes = gca;
                Func_SetGrid(Axes,opts)
                Func_SetAxesFont(Axes,opts)
                Func_SetBoundaries(opts)
                xlabel('$h$ [$\ $]','interpreter','latex')
                ylabel('$V$ [$mV$]','interpreter','latex')
                zlabel('$s$ [$\ $]','interpreter','latex')
                if isVisual, Func_SaveFigure(fig,sprintf('LAV_3D_C0e_EXP%i',iEXP),opts); end

%% SLOW MANIFOLD - VISUALIZATION: MANIFOLDS AND SIMULATION
% Visualization of the attracting and repelling slow submanifolds
% integrated with information related with the 3D simulations of the model.

VAR3DR = {'h','s','V'};
opts.zlim = [FS.V*1.05 FS.V*0.95];
opts.xlim = [FS.h*0.90 FS.h*1.05];
opts.ylim = [FS.s*0.95 FS.s*1.01];

fig = figure(); Func_VisualizeSlowManifold2(SB,'R',VAR3DR,0);
                Func_VisualizeSlowManifold2(SF,'A',VAR3DR,0);
                Func_VisualizeSimulationPS(fig,EXP,VAR3DR,CTRL,param.IApp);
                view(-75,15), Axes = gca;
                Func_SetGrid(Axes,opts)
                Func_SetAxesFont(Axes,opts)
                Func_SetBoundaries(opts)
                xlabel('$h$ [$\ $]','interpreter','latex')
                ylabel('$s$ [$\ $]','interpreter','latex')
                zlabel('$V$ [$mV$]','interpreter','latex')
                if isVisual, Func_SaveFigure(fig,sprintf('LAV_3D_C0e_EXP%i',iEXP),opts); end

